from Embedding.gf import *

def GF(input_file, output_file,matrix,rep_size=128, epoch=120, learning_rate=0.003, weight_decay=1.):
    

    np.savetxt(input_file,matrix,fmt='%d')
    t1 = time.time()
    g = Graph()
    print("Reading...")

    g.read_edgelist(filename=input_file, weighted=False, directed=False)
    model = GraphFactorization(graph=g, rep_size=rep_size, epoch=epoch, learning_rate=learning_rate, weight_decay=weight_decay)

    t2 = time.time()
    print(t2-t1)
    print("Saving embeddings...")
    model.save_embeddings(output_file)
    Embedding=np.loadtxt(output_file, dtype=float, skiprows=1)
    return Embedding



from .inits import *
import tensorflow as tf

flags = tf.app.flags
FLAGS = flags.FLAGS

# global unique layer ID dictionary for layer name assignment
_LAYER_UIDS = {}


def get_layer_uid(layer_name=''):
    """Helper function, assigns unique layer IDs."""
    if layer_name not in _LAYER_UIDS:
        _LAYER_UIDS[layer_name] = 1
        return 1
    else:
        _LAYER_UIDS[layer_name] += 1
        return _LAYER_UIDS[layer_name]


def sparse_dropout(x, keep_prob, noise_shape):
    """Dropout for sparse tensors."""
    random_tensor = keep_prob
    random_tensor += tf.random_uniform(noise_shape)
    dropout_mask = tf.cast(tf.floor(random_tensor), dtype=tf.bool)
    pre_out = tf.sparse_retain(x, dropout_mask)
    return pre_out * (1./keep_prob)


def dot(x, y, sparse=False):
    """Wrapper for tf.matmul (sparse vs dense)."""
    if sparse:
        res = tf.sparse_tensor_dense_matmul(x, y)
    else:
        res = tf.matmul(x, y)
    return res


class Layer(object):
    """Base layer class. Defines basic API for all layer objects.
    Implementation inspired by keras (http://keras.io).

    # Properties
        name: String, defines the variable scope of the layer.
        logging: Boolean, switches Tensorflow histogram logging on/off

    # Methods
        _call(inputs): Defines computation graph of layer
            (i.e. takes input, returns output)
        __call__(inputs): Wrapper for _call()
        _log_vars(): Log all variables
    """

    def __init__(self, **kwargs):
        allowed_kwargs = {'name', 'logging'}
        for kwarg in kwargs.keys():
            assert kwarg in allowed_kwargs, 'Invalid keyword argument: ' + kwarg
        name = kwargs.get('name')
        if not name:
            layer = self.__class__.__name__.lower()
            name = layer + '_' + str(get_layer_uid(layer))
        self.name = name
        self.vars = {}
        logging = kwargs.get('logging', False)
        self.logging = logging
        self.sparse_inputs = False

    def _call(self, inputs):
        return inputs

    def __call__(self, inputs):
        with tf.name_scope(self.name):
            if self.logging and not self.sparse_inputs:
                tf.summary.histogram(self.name + '/inputs', inputs)
            outputs = self._call(inputs)
            if self.logging:
                tf.summary.histogram(self.name + '/outputs', outputs)
            return outputs

    def _log_vars(self):
        for var in self.vars:
            tf.summary.histogram(self.name + '/vars/' + var, self.vars[var])


class Dense(Layer):
    """Dense layer."""

    def __init__(self, input_dim, output_dim, placeholders, dropout=0., sparse_inputs=False,
                 act=tf.nn.relu, bias=False, featureless=False, **kwargs):
        super(Dense, self).__init__(**kwargs)

        if dropout:
            self.dropout = placeholders['dropout']
        else:
            self.dropout = 0.

        self.act = act
        self.sparse_inputs = sparse_inputs
        self.featureless = featureless
        self.bias = bias

        # helper variable for sparse dropout
        self.num_features_nonzero = placeholders['num_features_nonzero']

        with tf.variable_scope(self.name + '_vars'):
            self.vars['weights'] = glorot([input_dim, output_dim],
                                          name='weights')
            if self.bias:
                self.vars['bias'] = zeros([output_dim], name='bias')

        if self.logging:
            self._log_vars()

    def _call(self, inputs):
        x = inputs

        # dropout
        if self.sparse_inputs:
            x = sparse_dropout(x, 1-self.dropout, self.num_features_nonzero)
        else:
            x = tf.nn.dropout(x, 1-self.dropout)

        # transform
        output = dot(x, self.vars['weights'], sparse=self.sparse_inputs)

        # bias
        if self.bias:
            output += self.vars['bias']

        return self.act(output)


class GraphConvolution(Layer):
    """Graph convolution layer."""

    def __init__(self, input_dim, output_dim, placeholders, dropout=0.,
                 sparse_inputs=False, act=tf.nn.relu, bias=False,
                 featureless=False, **kwargs):
        super(GraphConvolution, self).__init__(**kwargs)

        if dropout:
            self.dropout = placeholders['dropout']
        else:
            self.dropout = 0.

        self.act = act
        self.support = placeholders['support']
        self.sparse_inputs = sparse_inputs
        self.featureless = featureless
        self.bias = bias

        # helper variable for sparse dropout
        self.num_features_nonzero = placeholders['num_features_nonzero']

        with tf.variable_scope(self.name + '_vars'):
            for i in range(len(self.support)):
                self.vars['weights_' + str(i)] = glorot([input_dim, output_dim],
                                                        name='weights_' + str(i))
            if self.bias:
                self.vars['bias'] = zeros([output_dim], name='bias')

        if self.logging:
            self._log_vars()

    def _call(self, inputs):
        x = inputs

        # dropout
        if self.sparse_inputs:
            x = sparse_dropout(x, 1-self.dropout, self.num_features_nonzero)
        else:
            x = tf.nn.dropout(x, 1-self.dropout)

        # convolve
        supports = list()
        for i in range(len(self.support)):
            if not self.featureless:
                pre_sup = dot(x, self.vars['weights_' + str(i)],
                              sparse=self.sparse_inputs)
            else:
                pre_sup = self.vars['weights_' + str(i)]
            support = dot(self.support[i], pre_sup, sparse=True)
            supports.append(support)
        output = tf.add_n(supports)

        # bias
        if self.bias:
            output += self.vars['bias']

        return self.act(output)



import math
import numpy as np
from numpy import linalg as la
from sklearn.preprocessing import normalize
from .graph import *
from . import walker
import time

class GraRep(object):

    def __init__(self, graph, Kstep, dim):
        self.g = graph
        self.Kstep = Kstep
        assert dim % Kstep == 0
        self.dim = int(dim/Kstep)
        self.train()

    def getAdjMat(self):
        graph = self.g.G
        node_size = self.g.node_size
        look_up = self.g.look_up_dict
        adj = np.zeros((node_size, node_size))
        for edge in self.g.G.edges():
            adj[look_up[edge[0]]][look_up[edge[1]]] = 1.0
            adj[look_up[edge[1]]][look_up[edge[0]]] = 1.0
        # ScaleSimMat
        return np.matrix(adj/np.sum(adj, axis=1))

    def GetProbTranMat(self, Ak):
        probTranMat = np.log(Ak/np.tile(
            np.sum(Ak, axis=0), (self.node_size, 1))) \
            - np.log(1.0/self.node_size)
        probTranMat[probTranMat < 0] = 0
        probTranMat[probTranMat == np.nan] = 0
        return probTranMat

    def GetRepUseSVD(self, probTranMat, alpha):
        U, S, VT = la.svd(probTranMat)
        Ud = U[:, 0:self.dim]
        Sd = S[0:self.dim]
        return np.array(Ud)*np.power(Sd, alpha).reshape((self.dim))

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors.keys())
        fout.write("{} {}\n".format(node_num, self.Kstep*self.dim))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node, ' '.join([str(x) for x in vec])))
        fout.close()

    def train(self):
        self.adj = self.getAdjMat()
        self.node_size = self.adj.shape[0]
        self.Ak = np.matrix(np.identity(self.node_size))
        self.RepMat = np.zeros((self.node_size, int(self.dim*self.Kstep)))
        for i in range(self.Kstep):
            # print('Kstep =', i)
            self.Ak = np.dot(self.Ak, self.adj)
            probTranMat = self.GetProbTranMat(self.Ak)
            Rk = self.GetRepUseSVD(probTranMat, 0.5)
            Rk = normalize(Rk, axis=1, norm='l2')
            self.RepMat[:, self.dim*i:self.dim*(i+1)] = Rk[:, :]
        # get embeddings
        self.vectors = {}
        look_back = self.g.look_back_list
        for i, embedding in enumerate(self.RepMat):
            self.vectors[look_back[i]] = embedding



from __future__ import print_function
import time
from gensim.models import Word2Vec
from .graph import *


import random
import numpy as np
import multiprocessing
from . import walker


class Node2vec(object):

    def __init__(self, graph, path_length, num_paths, dim, p=1.0, q=1.0, dw=False, **kwargs):

        kwargs["workers"] = kwargs.get("workers", 1)
        if dw:
            kwargs["hs"] = 1
            p = 1.0
            q = 1.0

        self.graph = graph
        if dw:
            self.walker = walker.BasicWalker(graph, workers=kwargs["workers"])
        else:
            self.walker = walker.Walker(
                graph, p=p, q=q, workers=kwargs["workers"])
            # print("Preprocess transition probs...")
            self.walker.preprocess_transition_probs()
        sentences = self.walker.simulate_walks(
            num_walks=num_paths, walk_length=path_length)
        kwargs["sentences"] = sentences
        kwargs["min_count"] = kwargs.get("min_count", 0)
        kwargs["size"] = kwargs.get("size", dim)
        kwargs["sg"] = 1

        self.size = kwargs["size"]
        # print("Learning representation...")
        word2vec = Word2Vec(**kwargs)
        # word2vec = Word2Vec(vector_size=300, min_count=20, window=7, workers=1,epochs=15)
        self.vectors = {}
        for word in graph.G.nodes():
            self.vectors[word] = word2vec.wv[word]
        del word2vec

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors.keys())
        fout.write("{} {}\n".format(node_num, self.size))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node,
                                        ' '.join([str(x) for x in vec])))
        fout.close()



#coding=UTF-8
'''
Created on Nov 29, 2016

@author: ZWT
'''

import numpy as np
import time

def LP(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
    Matrix_similarity = np.dot(MatrixAdjacency_Train,MatrixAdjacency_Train)
    
    Parameter = 1
    Matrix_LP = np.dot(np.dot(MatrixAdjacency_Train,MatrixAdjacency_Train),MatrixAdjacency_Train) * Parameter
    
    Matrix_similarity = np.dot(Matrix_similarity,Matrix_LP)
    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity



from Embedding.lle import *
def lle(input_file, output_file, matrix,d=64):

    np.savetxt(input_file,matrix,fmt='%d')
    t1 = time()
    g = Graph()
    print("Reading...")

    g.read_edgelist(filename=input_file, weighted=False, directed=False)
    model = LLE(graph=g, d=d)


    t2 = time()
    print(t2-t1)
    print("Saving embeddings...")
    model.save_embeddings(output_file)
    Embedding=np.loadtxt(output_file, dtype=float, skiprows=1)
    return Embedding




#coding=UTF-8
'''
Created on 2016��11��19��

@author: ZWT
'''
import numpy as np
import time
def Calculation_AUC(MatrixAdjacency_Train,MatrixAdjacency_Test,Matrix_similarity,MaxNodeNum):
    AUC_TimeStart = time.clock()
    print ('    Calculation AUC......')
    AUCnum = 672400
    
    Matrix_similarity = np.triu(Matrix_similarity - Matrix_similarity * MatrixAdjacency_Train)
    Matrix_NoExist = np.ones(MaxNodeNum) - MatrixAdjacency_Train - MatrixAdjacency_Test - np.eye(MaxNodeNum)
     
    Test = np.triu(MatrixAdjacency_Test)
    NoExist = np.triu(Matrix_NoExist)
    
#     Test_num =len(np.argwhere(Test == 1))
#     NoExist_num = len(np.argwhere(NoExist == 1))
# # #   Test_num = np.nonzero(Test)[0].shape[0]
# # #   NoExist_num = np.nonzero(NoExist)[0].shape[0]

    Test_num = len(np.argwhere(Test == 1))
    NoExist_num = len(np.argwhere(NoExist == 1))
#     print '    Test_num：%d'%Test_num
#     print '    NoExist_num：%d'%NoExist_num
      
    Test_rd = [int(x) for index,x in enumerate((Test_num * np.random.rand(1,AUCnum))[0])]
    NoExist_rd = [int(x) for index,x in enumerate((NoExist_num * np.random.rand(1,AUCnum))[0])]
#     print '    Test_rd：'+str(Test_rd)
#     print '    Test_rd长度：'+str(len(Test_rd))
#     print '    Test_rd最大值：'+str(max(Test_rd))
#     print '    NoExist_rd：'+str(NoExist_rd)
#     print '    NoExist_rd长度：'+str(len(NoExist_rd))
    TestPre= Matrix_similarity * Test
    NoExistPre = Matrix_similarity * NoExist
    
    TestIndex = np.argwhere(Test == 1)
    Test_Data = np.array([TestPre[x[0],x[1]] for index,x in enumerate(TestIndex)]).T
    NoExistIndex = np.argwhere(NoExist == 1)
    NoExist_Data = np.array([NoExistPre[x[0],x[1]] for index,x in enumerate(NoExistIndex)]).T
#     print Test_Data
#     print Test_Data.shape
#     print NoExist_Data
#     print NoExist_Data.shape
    
    Test_rd = np.array([Test_Data[x] for index,x in enumerate(Test_rd)])
    NoExist_rd = np.array([NoExist_Data[x] for index,x in enumerate(NoExist_rd)])
#     print Test_rd
#     print Test_rd.shape
#     print NoExist_rd
#     print NoExist_rd.shape

#     aucArray = Test_rd - NoExist_rd
#     n1 = len(np.argwhere(aucArray > 0))
#     n2 = len(np.argwhere(aucArray == 0))
    n1,n2 = 0,0
    for num in range(AUCnum):
        if Test_rd[num] > NoExist_rd[num]:
            n1 += 1
        elif Test_rd[num] == NoExist_rd[num]:
            n2 += 0.5
        else:
            n1 += 0
    auc = float(n1+n2)/AUCnum
    print ('    AUC指标为：',auc)
    AUC_TimeEnd = time.clock()
    print ('    AUCTime：',AUC_TimeEnd - AUC_TimeStart)
    return auc


#177497
from utils import *
from Classifier.Genetic_Algorithm.KNeighborsClassifier import *
random.seed(123)
data = ReadCsv('feature.csv')
targets = ReadCsv('labels.csv')
targets = [int(i[0]) for i in targets]
data, targets=shuffle(data, targets)
optimize_type=['accuracy','f1','precision','recall','roc_auc']
best_param=Best_Param(data, targets,optimize_type[4])
model = KNeighborsClassifier(**best_param)

roc_auc = cross_val_score(model, data, targets,scoring='roc_auc', cv=5)
print("AUC=",roc_auc.mean())



from __future__ import print_function
import random
import math
import time
import numpy as np
from sklearn.linear_model import LogisticRegression
import tensorflow as tf
from .classify import Classifier, read_node_label
from . import walker
from .graph import *

class _LINE(object):

    def __init__(self, graph, rep_size=128, batch_size=1000, negative_ratio=5, order=3):
        self.cur_epoch = 0
        self.order = order
        self.g = graph
        self.node_size = graph.G.number_of_nodes()
        self.rep_size = rep_size
        self.batch_size = batch_size
        self.negative_ratio = negative_ratio

        self.gen_sampling_table()
        self.sess = tf.Session()
        cur_seed = random.getrandbits(32)
        initializer = tf.contrib.layers.xavier_initializer(
            uniform=False, seed=cur_seed)
        with tf.variable_scope("model", reuse=None, initializer=initializer):
            self.build_graph()
        self.sess.run(tf.global_variables_initializer())

    def build_graph(self):
        self.h = tf.placeholder(tf.int32, [None])
        self.t = tf.placeholder(tf.int32, [None])
        self.sign = tf.placeholder(tf.float32, [None])

        cur_seed = random.getrandbits(32)
        self.embeddings = tf.get_variable(name="embeddings"+str(self.order), shape=[
                                          self.node_size, self.rep_size], initializer=tf.contrib.layers.xavier_initializer(uniform=False, seed=cur_seed))
        self.context_embeddings = tf.get_variable(name="context_embeddings"+str(self.order), shape=[
                                                  self.node_size, self.rep_size], initializer=tf.contrib.layers.xavier_initializer(uniform=False, seed=cur_seed))
        # self.h_e = tf.nn.l2_normalize(tf.nn.embedding_lookup(self.embeddings, self.h), 1)
        # self.t_e = tf.nn.l2_normalize(tf.nn.embedding_lookup(self.embeddings, self.t), 1)
        # self.t_e_context = tf.nn.l2_normalize(tf.nn.embedding_lookup(self.context_embeddings, self.t), 1)
        self.h_e = tf.nn.embedding_lookup(self.embeddings, self.h)
        self.t_e = tf.nn.embedding_lookup(self.embeddings, self.t)
        self.t_e_context = tf.nn.embedding_lookup(
            self.context_embeddings, self.t)
        self.second_loss = -tf.reduce_mean(tf.log_sigmoid(
            self.sign*tf.reduce_sum(tf.multiply(self.h_e, self.t_e_context), axis=1)))
        self.first_loss = -tf.reduce_mean(tf.log_sigmoid(
            self.sign*tf.reduce_sum(tf.multiply(self.h_e, self.t_e), axis=1)))
        if self.order == 1:
            self.loss = self.first_loss
        else:
            self.loss = self.second_loss
        optimizer = tf.train.AdamOptimizer(0.001)
        self.train_op = optimizer.minimize(self.loss)

    def train_one_epoch(self):
        sum_loss = 0.0
        batches = self.batch_iter()
        batch_id = 0
        for batch in batches:
            h, t, sign = batch
            feed_dict = {
                self.h: h,
                self.t: t,
                self.sign: sign,
            }
            _, cur_loss = self.sess.run([self.train_op, self.loss], feed_dict)
            sum_loss += cur_loss
            batch_id += 1
        # print('epoch:{} sum of loss:{!s}'.format(self.cur_epoch, sum_loss))
        self.cur_epoch += 1

    def batch_iter(self):
        look_up = self.g.look_up_dict

        table_size = 1e8
        numNodes = self.node_size

        edges = [(look_up[x[0]], look_up[x[1]]) for x in self.g.G.edges()]

        data_size = self.g.G.number_of_edges()
        edge_set = set([x[0]*numNodes+x[1] for x in edges])
        shuffle_indices = np.random.permutation(np.arange(data_size))

        # positive or negative mod
        mod = 0
        mod_size = 1 + self.negative_ratio
        h = []
        t = []
        sign = 0

        start_index = 0
        end_index = min(start_index+self.batch_size, data_size)
        while start_index < data_size:
            if mod == 0:
                sign = 1.
                h = []
                t = []
                for i in range(start_index, end_index):
                    if not random.random() < self.edge_prob[shuffle_indices[i]]:
                        shuffle_indices[i] = self.edge_alias[shuffle_indices[i]]
                    cur_h = edges[shuffle_indices[i]][0]
                    cur_t = edges[shuffle_indices[i]][1]
                    h.append(cur_h)
                    t.append(cur_t)
            else:
                sign = -1.
                t = []
                for i in range(len(h)):
                    t.append(
                        self.sampling_table[random.randint(0, table_size-1)])

            yield h, t, [sign]
            mod += 1
            mod %= mod_size
            if mod == 0:
                start_index = end_index
                end_index = min(start_index+self.batch_size, data_size)

    def gen_sampling_table(self):
        table_size = 1e8
        power = 0.75
        numNodes = self.node_size

        # print("Pre-procesing for non-uniform negative sampling!")
        node_degree = np.zeros(numNodes)  # out degree

        look_up = self.g.look_up_dict
        for edge in self.g.G.edges():
            node_degree[look_up[edge[0]]
                        ] += self.g.G[edge[0]][edge[1]]["weight"]

        norm = sum([math.pow(node_degree[i], power) for i in range(numNodes)])

        self.sampling_table = np.zeros(int(table_size), dtype=np.uint32)

        p = 0
        i = 0
        for j in range(numNodes):
            p += float(math.pow(node_degree[j], power)) / norm
            while i < table_size and float(i) / table_size < p:
                self.sampling_table[i] = j
                i += 1

        data_size = self.g.G.number_of_edges()
        self.edge_alias = np.zeros(data_size, dtype=np.int32)
        self.edge_prob = np.zeros(data_size, dtype=np.float32)
        large_block = np.zeros(data_size, dtype=np.int32)
        small_block = np.zeros(data_size, dtype=np.int32)

        total_sum = sum([self.g.G[edge[0]][edge[1]]["weight"]
                         for edge in self.g.G.edges()])
        norm_prob = [self.g.G[edge[0]][edge[1]]["weight"] *
                     data_size/total_sum for edge in self.g.G.edges()]
        num_small_block = 0
        num_large_block = 0
        cur_small_block = 0
        cur_large_block = 0
        for k in range(data_size-1, -1, -1):
            if norm_prob[k] < 1:
                small_block[num_small_block] = k
                num_small_block += 1
            else:
                large_block[num_large_block] = k
                num_large_block += 1
        while num_small_block and num_large_block:
            num_small_block -= 1
            cur_small_block = small_block[num_small_block]
            num_large_block -= 1
            cur_large_block = large_block[num_large_block]
            self.edge_prob[cur_small_block] = norm_prob[cur_small_block]
            self.edge_alias[cur_small_block] = cur_large_block
            norm_prob[cur_large_block] = norm_prob[cur_large_block] + \
                norm_prob[cur_small_block] - 1
            if norm_prob[cur_large_block] < 1:
                small_block[num_small_block] = cur_large_block
                num_small_block += 1
            else:
                large_block[num_large_block] = cur_large_block
                num_large_block += 1

        while num_large_block:
            num_large_block -= 1
            self.edge_prob[large_block[num_large_block]] = 1
        while num_small_block:
            num_small_block -= 1
            self.edge_prob[small_block[num_small_block]] = 1

    def get_embeddings(self):
        vectors = {}
        embeddings = self.embeddings.eval(session=self.sess)
        # embeddings = self.sess.run(tf.nn.l2_normalize(self.embeddings.eval(session=self.sess), 1))
        look_back = self.g.look_back_list
        for i, embedding in enumerate(embeddings):
            vectors[look_back[i]] = embedding
        return vectors


class LINE(object):

    def __init__(self, graph, rep_size=128, batch_size=1000, epoch=10, negative_ratio=5, order=3, label_file=None, clf_ratio=0.5, auto_save=True):
        self.rep_size = rep_size
        self.order = order
        self.best_result = 0
        self.vectors = {}
        if order == 3:
            self.model1 = _LINE(graph, rep_size/2, batch_size,
                                negative_ratio, order=1)
            self.model2 = _LINE(graph, rep_size/2, batch_size,
                                negative_ratio, order=2)
            for i in range(epoch):
                self.model1.train_one_epoch()
                self.model2.train_one_epoch()
                if label_file:
                    self.get_embeddings()
                    X, Y = read_node_label(label_file)
                    # print("Training classifier using {:.2f}% nodes...".format(
                    #     clf_ratio*100))
                    clf = Classifier(vectors=self.vectors,
                                     clf=LogisticRegression())
                    result = clf.split_train_evaluate(X, Y, clf_ratio)

                    if result['macro'] > self.best_result:
                        self.best_result = result['macro']
                        if auto_save:
                            self.best_vector = self.vectors

        else:
            self.model = _LINE(graph, rep_size, batch_size,
                               negative_ratio, order=self.order)
            for i in range(epoch):
                self.model.train_one_epoch()
                if label_file:
                    self.get_embeddings()
                    X, Y = read_node_label(label_file)
                    # print("Training classifier using {:.2f}% nodes...".format(
                    #     clf_ratio*100))
                    clf = Classifier(vectors=self.vectors,
                                     clf=LogisticRegression())
                    result = clf.split_train_evaluate(X, Y, clf_ratio)

                    if result['macro'] > self.best_result:
                        self.best_result = result['macro']
                        if auto_save:
                            self.best_vector = self.vectors

        self.get_embeddings()
        if auto_save and label_file:
            self.vectors = self.best_vector

    def get_embeddings(self):
        self.last_vectors = self.vectors
        self.vectors = {}
        if self.order == 3:
            vectors1 = self.model1.get_embeddings()
            vectors2 = self.model2.get_embeddings()
            for node in vectors1.keys():
                self.vectors[node] = np.append(vectors1[node], vectors2[node])
        else:
            self.vectors = self.model.get_embeddings()

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors.keys())
        fout.write("{} {}\n".format(node_num, self.rep_size))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node,
                                        ' '.join([str(x) for x in vec])))
        fout.close()



import AdaBoostClassifier,CatBoostClassifier,KNeighborsClassifier,LightGBMClassifier,MLPClassifier,RandomForestClassifier,SVMClassifier,XGBClassifier
from sklearn.ensemble import AdaBoostClassifier as AdaBoost
from catboost import CatBoostClassifier as CatBoost
from catboost import  Pool
from sklearn.neighbors import KNeighborsClassifier as KNeighbors
from lightgbm import LGBMClassifier as LGBM
from sklearn.neural_network import MLPClassifier as MLP
from sklearn.ensemble import RandomForestClassifier as RandomForest
from sklearn.svm import SVC 
from  xgboost import XGBClassifier as XGB

from sklearn.model_selection import StratifiedKFold
import joblib
PATH='./AutoClassifierModel'
def k_fold_Bagging(data, targets,best_param,Flag,k):
    try:
        os.mkdir(PATH+'/'+Flag)
    except:
        pass
    cv = StratifiedKFold(n_splits=k)
    NUM=0
    for train, test in cv.split(data, targets):

        if Flag=='ada':
            model = AdaBoost(**best_param)
        if Flag=='cat':
            model = CatBoost(**best_param)
        if Flag=='knn':
            model = KNeighbors(**best_param) 
        if Flag=='gbm':
            model = LGBM(**best_param)
        if Flag=='mlp':
            model = MLP(**best_param)
        if Flag=='rf':
            model = RandomForest(**best_param)
        if Flag=='svm':
            model = SVC(**best_param)
        if Flag=='xg':
            model = XGB(**best_param)


        x_train=data[train]
        x_test = data[test]
        y_train=targets[train]
        y_test=targets[test]

        model.fit(x_train, y_train)

        if Flag=='ada':
            joblib.dump(model, PATH+'/'+Flag+'/'+str(NUM)+'.auto')
        if Flag=='cat':
         
            model.save_model(PATH+'/'+Flag+'/'+str(NUM)+'.auto')
        if Flag=='knn':
            joblib.dump(model, PATH+'/'+Flag+'/'+str(NUM)+'.auto')
        if Flag=='gbm':
            joblib.dump(model, PATH+'/'+Flag+'/'+str(NUM)+'.auto')
        if Flag=='mlp':
            joblib.dump(model, PATH+'/'+Flag+'/'+str(NUM)+'.auto')
        if Flag=='rf':
            joblib.dump(model, PATH+'/'+Flag+'/'+str(NUM)+'.auto')
        if Flag=='svm':
            joblib.dump(model, PATH+'/'+Flag+'/'+str(NUM)+'.auto')
        if Flag=='xg':
            
            model.save_model(PATH+'/'+Flag+'/'+str(NUM)+'.auto')


        joblib.dump(model, PATH+'/'+Flag+'/'+str(NUM)+'.auto')


    
def Stack_Ensembling(data, targets,k):

    Ensemble=[]
    List=['ada','cat','knn','gbm','mlp','rf','svm','xg']

    for Flag in List:
        part=[]
        for NUM in range(k):
            if Flag=='ada':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='cat':
                model=CatBoost
                model.load_model(PATH+'/'+Flag+'/'+str(NUM)+'.auto')
            if Flag=='knn':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='gbm':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='mlp':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='rf':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='svm':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='xg':
                model = XGB
                model.load_model(PATH+'/'+Flag+'/'+str(NUM)+'.auto')
   
            prob=model.predict_proba(data)[:, 1]
            part.append(prob)

        Ensemble.append(part)


        feature=[]
        for kk in range(len(targets)):
            part=[]
            for ii in Ensemble:

                for jj in ii:
                    part.append(jj[kk])

        
            feature.append(part)

        
    x_train, x_val, y_train, y_val = train_test_split(feature, targets, test_size=0.2)
    train_pool = Pool(x_train,y_train)
    val_pool = Pool(x_val, y_val)

    model  = CatBoostClassifier(task_type='CPU',use_best_model=True,loss_function='MultiClass',custom_loss=['Recall', 'Accuracy'],eval_metric='Accuracy')

    model.fit(train_pool, eval_set=val_pool,use_best_model=True, silent=False,plot=False)

    model.save_model(PATH+'/Stack_Ensembling/cat.auto')




def fit(data, targets,optimize_type,k):

    try:
        os.mkdir(PATH)
    except:
        pass
    adaboost_best_param=AdaBoostClassifier.Best_Param(data, targets,optimize_type)
    catboost_best_param=CatBoostClassifier.Best_Param(data, targets,optimize_type)
    KNN_best_param=KNeighborsClassifier.Best_Param(data, targets,optimize_type)
    lightGBM_best_param=LightGBMClassifier.Best_Param(data, targets,optimize_type)
    MLP_best_param=MLPClassifier.Best_Param(data, targets,optimize_type)
    RF_best_param=RandomForestClassifier.Best_Param(data, targets,optimize_type)
    SVM_best_param=SVMClassifier.Best_Param(data, targets,optimize_type)
    xgboost_best_param=XGBClassifier.Best_Param(data, targets,optimize_type)

    k_fold_Bagging(data, targets,adaboost_best_param,'ada',k)
    k_fold_Bagging(data, targets,catboost_best_param,'cat',k)
    k_fold_Bagging(data, targets,KNN_best_param,'knn',k)
    k_fold_Bagging(data, targets,lightGBM_best_param,'gbm',k)
    k_fold_Bagging(data, targets,MLP_best_param,'mlp',k)
    k_fold_Bagging(data, targets,RF_best_param,'rf',k)
    k_fold_Bagging(data, targets,SVM_best_param,'svm',k)
    k_fold_Bagging(data, targets,xgboost_best_param,'xg',k)

    Stack_Ensembling(data, targets,k)

def predict_proba(data):

    Ensemble=[]
    List=['ada','cat','knn','gbm','mlp','rf','svm','xg']

    for Flag in List:
        part=[]
        for NUM in range(k):
            if Flag=='ada':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='cat':
                model=CatBoost
                model.load_model(PATH+'/'+Flag+'/'+str(NUM)+'.auto')
            if Flag=='knn':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='gbm':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='mlp':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='rf':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='svm':
                model = joblib.load(PATH+'/'+Flag+'/'+str(NUM)+'.auto') 
            if Flag=='xg':
                model = XGB
                model.load_model(PATH+'/'+Flag+'/'+str(NUM)+'.auto')
   
            prob=model.predict_proba(data)[:, 1]
            part.append(prob)

        Ensemble.append(part)


        feature=[]
        for kk in range(len(data)):
            part=[]
            for ii in Ensemble:

                for jj in ii:
                    part.append(jj[kk])

        
            feature.append(part)


        model = CatBoost()
        model.load_model(PATH+'/'+Flag+'/'+str(NUM)+'.auto')
        prob = model.predict_proba(feature)

        return prob


from .layers import *
from .metrics import *

flags = tf.app.flags
FLAGS = flags.FLAGS


class Model(object):
    def __init__(self, **kwargs):
        allowed_kwargs = {'name', 'logging'}
        for kwarg in kwargs.keys():
            assert kwarg in allowed_kwargs, 'Invalid keyword argument: ' + kwarg
        name = kwargs.get('name')
        if not name:
            name = self.__class__.__name__.lower()
        self.name = name

        logging = kwargs.get('logging', False)
        self.logging = logging

        self.vars = {}
        self.placeholders = {}

        self.layers = []
        self.activations = []

        self.inputs = None
        self.outputs = None

        self.loss = 0
        self.accuracy = 0
        self.optimizer = None
        self.opt_op = None

    def _build(self):
        raise NotImplementedError

    def build(self):
        """ Wrapper for _build() """
        with tf.variable_scope(self.name):
            self._build()

        # Build sequential layer model
        self.activations.append(self.inputs)
        for layer in self.layers:
            hidden = layer(self.activations[-1])
            self.activations.append(hidden)
        self.outputs = self.activations[-1]

        # Store model variables for easy access
        variables = tf.get_collection(
            tf.GraphKeys.GLOBAL_VARIABLES, scope=self.name)
        self.vars = {var.name: var for var in variables}

        # Build metrics
        self._loss()
        self._accuracy()

        self.opt_op = self.optimizer.minimize(self.loss)

    def predict(self):
        pass

    def _loss(self):
        raise NotImplementedError

    def _accuracy(self):
        raise NotImplementedError

    def save(self, sess=None):
        if not sess:
            raise AttributeError("TensorFlow session not provided.")
        saver = tf.train.Saver(self.vars)
        save_path = saver.save(sess, "tmp/%s.ckpt" % self.name)
        print("Model saved in file: %s" % save_path)

    def load(self, sess=None):
        if not sess:
            raise AttributeError("TensorFlow session not provided.")
        saver = tf.train.Saver(self.vars)
        save_path = "tmp/%s.ckpt" % self.name
        saver.restore(sess, save_path)
        print("Model restored from file: %s" % save_path)


class MLP(Model):
    def __init__(self, placeholders, input_dim, **kwargs):
        super(MLP, self).__init__(**kwargs)

        self.inputs = placeholders['features']
        self.input_dim = input_dim
        # self.input_dim = self.inputs.get_shape().as_list()[1]  # To be supported in future Tensorflow versions
        self.output_dim = placeholders['labels'].get_shape().as_list()[1]
        self.placeholders = placeholders

        self.optimizer = tf.train.AdamOptimizer(
            learning_rate=FLAGS.learning_rate)

        self.build()

    def _loss(self):
        # Weight decay loss
        for var in self.layers[0].vars.values():
            self.loss += FLAGS.weight_decay * tf.nn.l2_loss(var)

        # Cross entropy error
        self.loss += masked_softmax_cross_entropy(self.outputs, self.placeholders['labels'],
                                                  self.placeholders['labels_mask'])

    def _accuracy(self):
        self.accuracy = masked_accuracy(self.outputs, self.placeholders['labels'],
                                        self.placeholders['labels_mask'])

    def _build(self):
        self.layers.append(Dense(input_dim=self.input_dim,
                                 output_dim=FLAGS.hidden1,
                                 placeholders=self.placeholders,
                                 act=tf.nn.relu,
                                 dropout=True,
                                 sparse_inputs=True,
                                 logging=self.logging))

        self.layers.append(Dense(input_dim=FLAGS.hidden1,
                                 output_dim=self.output_dim,
                                 placeholders=self.placeholders,
                                 act=lambda x: x,
                                 dropout=True,
                                 logging=self.logging))

    def predict(self):
        return tf.nn.softmax(self.outputs)


class GCN(Model):
    def __init__(self, placeholders, input_dim, hidden1, weight_decay, **kwargs):
        super(GCN, self).__init__(**kwargs)

        self.inputs = placeholders['features']
        self.hidden1 = hidden1
        self.weight_decay = weight_decay
        self.input_dim = input_dim
        # self.input_dim = self.inputs.get_shape().as_list()[1]  # To be supported in future Tensorflow versions
        self.output_dim = placeholders['labels'].get_shape().as_list()[1]
        self.placeholders = placeholders

        self.optimizer = tf.train.AdamOptimizer(learning_rate=0.01)

        self.build()

    def _loss(self):
        # Weight decay loss
        for var in self.layers[0].vars.values():
            self.loss += self.weight_decay * tf.nn.l2_loss(var)

        # Cross entropy error
        self.loss += masked_softmax_cross_entropy(self.outputs, self.placeholders['labels'],
                                                  self.placeholders['labels_mask'])

    def _accuracy(self):
        self.accuracy = masked_accuracy(self.outputs, self.placeholders['labels'],
                                        self.placeholders['labels_mask'])

    def _build(self):

        self.layers.append(GraphConvolution(input_dim=self.input_dim,
                                            output_dim=self.hidden1,
                                            placeholders=self.placeholders,
                                            act=tf.nn.relu,
                                            dropout=True,
                                            sparse_inputs=True,
                                            logging=self.logging))

        self.layers.append(GraphConvolution(input_dim=self.hidden1,
                                            output_dim=self.output_dim,
                                            placeholders=self.placeholders,
                                            act=lambda x: x,
                                            dropout=True,
                                            logging=self.logging))

    def predict(self):
        return tf.nn.softmax(self.outputs)



import numpy as np
import networkx as nx
from .graph import *
from . import walker
import time


class LaplacianEigenmaps(object):
    def __init__(self, graph, rep_size=128):
        self.g = graph
        self.node_size = self.g.G.number_of_nodes()
        self.rep_size = rep_size
        self.adj_mat = nx.to_numpy_array(self.g.G)
        self.vectors = {}
        self.embeddings = self.get_train()
        look_back = self.g.look_back_list

        for i, embedding in enumerate(self.embeddings):
            self.vectors[look_back[i]] = embedding

    def getAdj(self):
        node_size = self.g.node_size
        look_up = self.g.look_up_dict
        adj = np.zeros((node_size, node_size))
        for edge in self.g.G.edges():
            adj[look_up[edge[0]]][look_up[edge[1]]] = self.g.G[edge[0]][edge[1]]['weight']
        return adj

    def getLap(self):
        degree_mat = np.diagflat(np.sum(self.adj_mat, axis=1))
        deg_trans = np.diagflat(np.reciprocal(np.sqrt(np.sum(self.adj_mat, axis=1))))
        deg_trans = np.nan_to_num(deg_trans)
        L = degree_mat-self.adj_mat

        # eye = np.eye(self.node_size)
        norm_lap_mat = np.matmul(np.matmul(deg_trans, L), deg_trans)
        return norm_lap_mat

    def get_train(self):
        lap_mat = self.getLap()
        w, vec = np.linalg.eigh(lap_mat)

        start = 0
        for i in range(self.node_size):
            if w[i] > 1e-10:
                start = i
                break
        vec = vec[:, start:start+self.rep_size]

        return vec

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors)
        fout.write("{} {}\n".format(node_num, self.rep_size))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node, ' '.join([str(x) for x in vec])))
        fout.close()




# -*- coding: utf-8 -*-

import geatpy as ea 
import numpy as np
import geatpy as ea
from sklearn import svm
from sklearn import preprocessing
from sklearn.model_selection import cross_val_score
from multiprocessing.dummy import Pool as ThreadPool
class MyProblem(ea.Problem): 
    def __init__(self,datas,data_targets,optimize_type):
        name = 'MyProblem' 
        M = 1
        maxormins = [-1] 
        Dim = 2 
        varTypes = [0, 0] 
        lb = [2**(-8)] * Dim 
        ub = [2**8] * Dim
        lbin = [1] * Dim 
        ubin = [1] * Dim       
        ea.Problem.__init__(self, name, M, maxormins, Dim, varTypes, lb, ub, lbin, ubin)
     
        
        self.data = datas 
        self.dataTarget = data_targets
        self.optimize_type = optimize_type
    
    def aimFunc(self, pop): 
        Vars = pop.Phen 
        pop.ObjV = np.zeros((pop.sizes, 1)) 
        def subAimFunc(i):
            C = Vars[i, 0]
            G = Vars[i, 1]
            svc = svm.SVC(C=C, kernel='rbf', gamma=G).fit(self.data, self.dataTarget) 
            scores = cross_val_score(svc, self.data, self.dataTarget, scoring=self.optimize_type,cv=5) 
            pop.ObjV[i] = scores.mean() 
        pool = ThreadPool(2) 
        pool.map(subAimFunc, list(range(pop.sizes)))
   




def Best_Param(datas,data_targets,optimize_type):


    problem = MyProblem(datas,data_targets,optimize_type) 


    Encoding = 'RI'       
    NIND = 2             
    Field = ea.crtfld(Encoding, problem.varTypes, problem.ranges, problem.borders) 
    population = ea.Population(Encoding, Field, NIND) 
    
    myAlgorithm = ea.soea_DE_rand_1_bin_templet(problem, population) 
    myAlgorithm.MAXGEN = 10 
    myAlgorithm.trappedValue = 1e-6 
    myAlgorithm.maxTrappedCount = 10 
    myAlgorithm.logTras = 1  
    myAlgorithm.verbose = True  
    myAlgorithm.drawing = 1  

    [BestIndi, population] = myAlgorithm.run()  
    BestIndi.save()  
 
    print('Timed：%f s' % myAlgorithm.passTime)
    
    if BestIndi.sizes != 0:
        print('The optimal objective function value is:%s' % BestIndi.ObjV[0][0])
        print('The optimal parameter value is:')
        for i in range(BestIndi.Phen.shape[1]):
            print(BestIndi.Phen[0, i])
    else:
        print('No feasible solution was found.')


    best_param={'C':BestIndi.Phen[0][0],
                'gamma':BestIndi.Phen[0][1],
                'cache_size':3000,
               }
    return best_param


from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.model_selection import cross_val_predict
from sklearn.model_selection import KFold
from sklearn.model_selection import cross_val_score
from sklearn.metrics import roc_curve,auc,precision_recall_curve,accuracy_score
import joblib
import numpy as np
from sklearn.svm import SVC
from bayes_opt import BayesianOptimization
from bayes_opt.util import Colours
random.seed(123)
def rfc_cv(C,gamma, data, targets,optimize_type):
    estimator = SVC(
        C=C,
        gamma=gamma
    )
    cval = cross_val_score(estimator, data, targets,
                           scoring=optimize_type, cv=5)
    return cval.mean()



def Best_Param(data, targets,optimize_type):
    def rfc_crossval(C,gamma):
        return rfc_cv(
            C=C,
            gamma=gamma,
            data=data,
            targets=targets,
            optimize_type=optimize_type,
        )

    optimizer = BayesianOptimization(
        f=rfc_crossval,
        pbounds={
            "C": (0, 100),
            "gamma": (1e-8, 0.99999),
            
        },
        random_state=1234,
        verbose=2
    )
    optimizer.maximize(n_iter=30)
    params=optimizer.max['params']
    modify_para=list(params.values())
    best_param={'C':modify_para[0],
                'gamma':modify_para[1],
                'cache_size':3000,
               }
    print(best_param)
    print(round(optimizer.max['target'],4))
    return best_param







from Embedding.line import *
def Line(input_file, output_file, matrix, rep_size=128, batch_size=1000, epoch=10, negative_ratio=5, order=3, label_file=None, clf_ratio=0.5, auto_save=True):

    np.savetxt(input_file,matrix,fmt='%d')
    t1 = time.time()
    g = Graph()
    print("Reading...")

    g.read_edgelist(filename=input_file, weighted=False, directed=False)
    model = LINE(g, rep_size=rep_size, batch_size=batch_size, epoch=epoch, 
                negative_ratio=negative_ratio, order=order, label_file=label_file, 
                clf_ratio=clf_ratio, auto_save=auto_save)

    t2 = time.time()
    print(t2-t1)
    print("Saving embeddings...")
    model.save_embeddings(output_file)
    Embedding=np.loadtxt(output_file, dtype=float, skiprows=1)
    return Embedding



import numpy as np
from .utils import *
from . import models
import time
import scipy.sparse as sp
import tensorflow as tf


class GCN(object):

    def __init__(self, graph, learning_rate=0.01, epochs=200,
                 hidden1=16, dropout=0.5, weight_decay=5e-4, early_stopping=10,
                 max_degree=3, clf_ratio=0.1):
        """
                        learning_rate: Initial learning rate
                        epochs: Number of epochs to train
                        hidden1: Number of units in hidden layer 1
                        dropout: Dropout rate (1 - keep probability)
                        weight_decay: Weight for L2 loss on embedding matrix
                        early_stopping: Tolerance for early stopping (# of epochs)
                        max_degree: Maximum Chebyshev polynomial degree
        """
        self.graph = graph
        self.clf_ratio = clf_ratio
        self.learning_rate = learning_rate
        self.epochs = epochs
        self.hidden1 = hidden1
        self.dropout = dropout
        self.weight_decay = weight_decay
        self.early_stopping = early_stopping
        self.max_degree = max_degree

        self.preprocess_data()
        self.build_placeholders()
        # Create model
        self.model = models.GCN(
            self.placeholders, input_dim=self.features[2][1], hidden1=self.hidden1, weight_decay=self.weight_decay, logging=True)
        # Initialize session
        self.sess = tf.Session()
        # Init variables
        self.sess.run(tf.global_variables_initializer())

        cost_val = []

        # Train model
        for epoch in range(self.epochs):

            t = time.time()
            # Construct feed dictionary
            feed_dict = self.construct_feed_dict(self.train_mask)
            feed_dict.update({self.placeholders['dropout']: self.dropout})

            # Training step
            outs = self.sess.run(
                [self.model.opt_op, self.model.loss, self.model.accuracy], feed_dict=feed_dict)

            # Validation
            cost, acc, duration = self.evaluate(self.val_mask)
            cost_val.append(cost)

            # Print results
            print("Epoch:", '%04d' % (epoch + 1), "train_loss=", "{:.5f}".format(outs[1]),
                  "train_acc=", "{:.5f}".format(
                outs[2]), "val_loss=", "{:.5f}".format(cost),
                "val_acc=", "{:.5f}".format(acc), "time=", "{:.5f}".format(time.time() - t))

            if epoch > self.early_stopping and cost_val[-1] > np.mean(cost_val[-(self.early_stopping+1):-1]):
                print("Early stopping...")
                break
        print("Optimization Finished!")

        # Testing
        test_cost, test_acc, test_duration = self.evaluate(self.test_mask)
        print("Test set results:", "cost=", "{:.5f}".format(test_cost),
              "accuracy=", "{:.5f}".format(test_acc), "time=", "{:.5f}".format(test_duration))

    # Define model evaluation function

    def evaluate(self, mask):
        t_test = time.time()
        feed_dict_val = self.construct_feed_dict(mask)
        outs_val = self.sess.run(
            [self.model.loss, self.model.accuracy], feed_dict=feed_dict_val)
        return outs_val[0], outs_val[1], (time.time() - t_test)

    def build_placeholders(self):
        num_supports = 1
        self.placeholders = {
            'support': [tf.sparse_placeholder(tf.float32) for _ in range(num_supports)],
            'features': tf.sparse_placeholder(tf.float32, shape=tf.constant(self.features[2], dtype=tf.int64)),
            'labels': tf.placeholder(tf.float32, shape=(None, self.labels.shape[1])),
            'labels_mask': tf.placeholder(tf.int32),
            'dropout': tf.placeholder_with_default(0., shape=()),
            # helper variable for sparse dropout
            'num_features_nonzero': tf.placeholder(tf.int32)
        }

    def build_label(self):
        g = self.graph.G
        look_up = self.graph.look_up_dict
        labels = []
        label_dict = {}
        label_id = 0
        for node in g.nodes():
            labels.append((node, g.nodes[node]['label']))
            for l in g.nodes[node]['label']:
                if l not in label_dict:
                    label_dict[l] = label_id
                    label_id += 1
        self.labels = np.zeros((len(labels), label_id))
        self.label_dict = label_dict
        for node, l in labels:
            node_id = look_up[node]
            for ll in l:
                l_id = label_dict[ll]
                self.labels[node_id][l_id] = 1

    def build_train_val_test(self):
        """
            build train_mask test_mask val_mask
        """
        train_precent = self.clf_ratio
        training_size = int(train_precent * self.graph.G.number_of_nodes())
        state = np.random.get_state()
        np.random.seed(0)
        shuffle_indices = np.random.permutation(
            np.arange(self.graph.G.number_of_nodes()))
        np.random.set_state(state)

        look_up = self.graph.look_up_dict
        g = self.graph.G

        def sample_mask(begin, end):
            mask = np.zeros(g.number_of_nodes())
            for i in range(begin, end):
                mask[shuffle_indices[i]] = 1
            return mask

        # nodes_num = len(self.labels)
        # self.train_mask = sample_mask('train', nodes_num)
        # self.val_mask = sample_mask('valid', nodes_num)
        # self.test_mask = sample_mask('test', nodes_num)
        self.train_mask = sample_mask(0, training_size-100)
        self.val_mask = sample_mask(training_size-100, training_size)
        self.test_mask = sample_mask(training_size, g.number_of_nodes())

    def preprocess_data(self):
        """
            adj, features, y_train, y_val, y_test, train_mask, val_mask, test_mask
            y_train, y_val, y_test can merge to y
        """
        g = self.graph.G
        look_back = self.graph.look_back_list
        self.features = np.vstack([g.nodes[look_back[i]]['feature']
                                   for i in range(g.number_of_nodes())])
        self.features = preprocess_features(self.features)
        self.build_label()
        self.build_train_val_test()
        adj = nx.adjacency_matrix(g)  # the type of graph
        self.support = [preprocess_adj(adj)]

    def construct_feed_dict(self, labels_mask):
        """Construct feed dictionary."""
        feed_dict = dict()
        feed_dict.update({self.placeholders['labels']: self.labels})
        feed_dict.update({self.placeholders['labels_mask']: labels_mask})
        feed_dict.update({self.placeholders['features']: self.features})
        feed_dict.update(
            {self.placeholders['support'][i]: self.support[i] for i in range(len(self.support))})
        feed_dict.update(
            {self.placeholders['num_features_nonzero']: self.features[1].shape})
        return feed_dict



#coding=UTF-8
'''
Created on 2016��11��22��

@author: ZWT
'''
import numpy as np
import time

def Jaccavrd(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
    
    Matrix_similarity = np.dot(MatrixAdjacency_Train,MatrixAdjacency_Train)
    
    deg_row = sum(MatrixAdjacency_Train)
    deg_row.shape = (deg_row.shape[0],1)
    deg_row_T = deg_row.T
    tempdeg = deg_row + deg_row_T
    temp = tempdeg - Matrix_similarity
    
    Matrix_similarity = Matrix_similarity / temp
    
    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity

# import numpy.matlib as matlab
# def spones(Matrix):
#     MatrixIndex = np.argwhere(Matrix != 0)
#     for index in range(len(MatrixIndex)):
#         Matrix[MatrixIndex[index,0],MatrixIndex[index,1]] = 1
#     return Matrix
#
# def Jaccavrd(MatrixAdjacency_Train):
#     Matrix_similarity = np.dot(MatrixAdjacency_Train,MatrixAdjacency_Train)
#     
#     deg_row = matlab.repmat(matlab.sum(MatrixAdjacency_Train,1), matlab.size(MatrixAdjacency_Train, 1), 1)
#     deg_row = deg_row * spones(Matrix_similarity)
#     deg_row = np.triu(deg_row) + np.triu(deg_row.T)
#     temp = (deg_row * spones(Matrix_similarity)) - Matrix_similarity
#     
#     Matrix_similarity = Matrix_similarity / temp
#     np.seterr(divide='ignore', invalid='ignore')
#     print np.isnan(Matrix_similarity)
#     Matrix_similarity = np.nan_to_num(Matrix_similarity)
#     print np.isnan(Matrix_similarity)
#     return Matrix_similarity


import networkx as nx
import numpy as np
import scipy.io as sio
import scipy.sparse as sp
import scipy.sparse.linalg as lg
from . import graph as g
import tensorflow as tf
from sklearn.preprocessing import normalize
from .graph import *
from . import walker
import time



class HOPE(object):
    def __init__(self, graph, d):
        '''
          d: representation vector dimension
        '''
        self._d = d
        self._graph = graph.G
        self.g = graph
        self._node_num = graph.node_size
        self.learn_embedding()

    def learn_embedding(self):

        graph = self.g.G
        A = nx.to_numpy_matrix(graph)

        # self._beta = 0.0728

        # M_g = np.eye(graph.number_of_nodes()) - self._beta * A
        # M_l = self._beta * A

        M_g = np.eye(graph.number_of_nodes())
        M_l = np.dot(A, A)

        S = np.dot(np.linalg.inv(M_g), M_l)
        # s: \sigma_k
        u, s, vt = lg.svds(S, k=self._d // 2)
        sigma = np.diagflat(np.sqrt(s))
        X1 = np.dot(u, sigma)
        X2 = np.dot(vt.T, sigma)
        # self._X = X2
        self._X = np.concatenate((X1, X2), axis=1)

    @property
    def vectors(self):
        vectors = {}
        look_back = self.g.look_back_list
        for i, embedding in enumerate(self._X):
            vectors[look_back[i]] = embedding
        return vectors

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors.keys())
        fout.write("{} {}\n".format(node_num, self._d))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node,
                                        ' '.join([str(x) for x in vec])))
        fout.close()



from __future__ import print_function
import numpy as np
import tensorflow as tf
import networkx as nx
import time
from .graph import *
from . import walker


__author__ = "Wang Binlu"
__email__ = "wblmail@whu.edu.cn"


class GraphFactorization(object):
    def __init__(self, graph, rep_size=128, epoch=120, learning_rate=0.003, weight_decay=1.):
        self.g = graph

        self.node_size = graph.G.number_of_nodes()
        self.rep_size = rep_size
        self.max_iter = epoch
        self.lr = learning_rate
        self.lamb = weight_decay
        self.sess = tf.Session()
        self.adj_mat = self.getAdj()
        self.vectors = {}

        self.embeddings = self.get_train()

        look_back = self.g.look_back_list

        for i, embedding in enumerate(self.embeddings):
            self.vectors[look_back[i]] = embedding

    def getAdj(self):
        node_size = self.g.node_size
        look_up = self.g.look_up_dict
        adj = np.zeros((node_size, node_size))
        for edge in self.g.G.edges():
            adj[look_up[edge[0]]][look_up[edge[1]]] = self.g.G[edge[0]][edge[1]]['weight']
        return adj

    def get_train(self):

        adj_mat = self.adj_mat

        mat_mask = 1.*(adj_mat > 0)

        _embeddings = tf.Variable(tf.contrib.layers.xavier_initializer()([self.node_size, self.rep_size]),
                                  dtype=tf.float32, name='embeddings')

        Adj = tf.placeholder(tf.float32, [self.node_size, self.node_size], name='adj_mat')
        AdjMask = tf.placeholder(tf.float32, [self.node_size, self.node_size], name='adj_mask')

        cost = tf.reduce_sum(
                tf.square(Adj - tf.matmul(_embeddings, tf.transpose(_embeddings))*AdjMask)) + \
               self.lamb * tf.reduce_sum(tf.square(_embeddings))

        optimizer = tf.train.AdamOptimizer(self.lr)
        train_op = optimizer.minimize(cost)

        init = tf.global_variables_initializer()
        self.sess.run(init)

        # print("total iter: %i" % self.max_iter)
        for step in range(self.max_iter):
            self.sess.run(train_op, feed_dict={Adj: adj_mat, AdjMask: mat_mask})
            # if step % 50 == 0:
            #     print("step %i: cost: %g" % (step, self.sess.run(cost, feed_dict={Adj: adj_mat, AdjMask: mat_mask})))
        return self.sess.run(_embeddings)

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors)
        fout.write("{} {}\n".format(node_num, self.rep_size))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node, ' '.join([str(x) for x in vec])))
        fout.close()



#178244
from autobiotool.utils import *
from Classifier.Genetic_Algorithm.SVMClassifier import *
from sklearn.metrics import roc_curve, auc
import sklearn
random.seed(123)
data = ReadCsv('feature.csv')
targets = ReadCsv('labels.csv')
targets = [int(i[0]) for i in targets]
data, targets=shuffle(data, targets)
# cls = autosklearn.classification.AutoSklearnClassifier( time_left_for_this_task=720, per_run_time_limit=190, ml_memory_limit=10000)
X_train, X_test, y_train, y_test = sklearn.model_selection.train_test_split(data, targets, random_state=1)
y_train=[[i] for i in y_train]



from autogluon.tabular import TabularDataset, TabularPredictor



label = 256  # specifies which column do we want to predict

save_path = 'ag_models/'  # where to save trained models

Train=np.hstack((X_train,y_train))

train_data = pd.DataFrame(Train)
X_test = pd.DataFrame(X_test)





#predictor = TabularPredictor(label=label, path=save_path).fit(train_data)

#print(X_test.head())

predictor = TabularPredictor.load(save_path)  # Unnecessary, we reload predictor just to demonstrate how to load previously-trained predictor from file
y_prob = predictor.predict_proba(X_test)
y_prob=[i[1] for i in y_prob.values.tolist()]

fpr, tpr, thresholds = roc_curve(y_test, y_prob)
roc_auc = auc(fpr, tpr)
print(roc_auc)


import pandas as pd
import numpy as np
from rdkit import Chem
from rdkit.Chem import AllChem


drug = pd.read_csv('./data/drug_smiles.csv')
smiles = drug['smiles'].values
mols = [Chem.MolFromSmiles(x) for x in smiles]
fp = [AllChem.GetMorganFingerprintAsBitVect(x,2,nBits=1024,) for x in mols]
fp_array = np.array(fp)
np.savetxt('./data/morganfp.txt',fp_array,delimiter=',')



#coding=UTF-8
'''
Created on 2016��11��20��

@author: ZWT
'''
import time
import os
import Initialize
import Evaluation_Indicators.AUC


import similarity_indicators.CommonNeighbor
import similarity_indicators.Salton
import similarity_indicators.Jaccard
import similarity_indicators.Sorenson
import similarity_indicators.HPI
import similarity_indicators.HDI
import similarity_indicators.LHN_I
import similarity_indicators.PA
import similarity_indicators.AA
import similarity_indicators.RA

import similarity_indicators.LP
import similarity_indicators.Katz

import similarity_indicators.ACT
import similarity_indicators.Cos

startTime = time.clock()
#初始化训练测试集合
READEME1 = '''Please choose a Set:
    PPI            1
    NS             2
    Grid           3
    PB             4
    INT            5
    USAir          6'''
print(READEME1)
Set = int(input('Input Set:'))
if Set == 1:
    NetFile = u'Data/PPI.txt'
    NetName = 'PPI'
elif Set == 2:
    NetFile = u'Data/NS.txt'
    NetName = 'NS'
elif Set == 3:
    NetFile = u'Data/Grid.txt'
    NetName = 'Grid'
elif Set == 4:
    NetFile = u'Data/PB.txt'
    NetName = 'PB'
elif Set == 5:
    NetFile = u'Data/INT.txt'
    NetName = 'INT'
elif Set == 6:
    NetFile = u'Data/USAir.txt'
    NetName = 'USAir'
else:
    print('Input Error')
     
print("\nLink Prediction start：\n")
TrainFile_Path = 'Data\\'+NetName+'\\Train.txt'
if os.path.exists(TrainFile_Path):
    Train_File = 'Data\\'+NetName+'\\Train.txt'
    Test_File = 'Data\\'+NetName+'\\Test.txt'
    MatrixAdjacency_Train,MatrixAdjacency_Test,MaxNodeNum = Initialize.Init2(Test_File, Train_File)
else:
    MatrixAdjacency_Net,MaxNodeNum = Initialize.Init(NetFile)
    MatrixAdjacency_Train,MatrixAdjacency_Test = Initialize.Divide(NetFile, MatrixAdjacency_Net, MaxNodeNum,NetName)

#计算相似性矩阵
# READEME = '''\nPlease choose a method:
#     CN            0
#     Salton        1
#     Jaccard       2
#     Sorenson      3
#     HPI           4
#     HDI           5
#     LHN-I         6
#     PA            7
#     AA            8
#     RA            9
#     LP            10
#     Katz          11
#     ACT           12
#     RWR           13'''
# print READEME
# Method = int(raw_input('Input Method:'))
similarity_StartTime = time.clock()

# Matrix_similarity = similarity_indicators.Cos.ACT(MatrixAdjacency_Train)

for Method in range(14):
    if Method == 0:
        print('----------SIM----------基于局部信息----------SIM----------')
        print('----------Cn----------')
        Matrix_similarity = similarity_indicators.CommonNeighbor.Cn(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 1:
        print('----------Salton----------')
        Matrix_similarity = similarity_indicators.Salton.Salton(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 2:
        print ('----------Jaccard----------')
        Matrix_similarity = similarity_indicators.Jaccard.Jaccavrd(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 3:
        print ('----------Sorenson----------')
        Matrix_similarity = similarity_indicators.Sorenson.Sorenson(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 4:
        print ('----------HPI----------')
        Matrix_similarity = similarity_indicators.HPI.HPI(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 5:
        print ('----------HDI----------')
        Matrix_similarity = similarity_indicators.HDI.HDI(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 6:
        print ('----------LHN-1----------')
        Matrix_similarity = similarity_indicators.LHN_I.LHN_I(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 7:
        print ('----------PA----------')
        Matrix_similarity = similarity_indicators.PA.PA(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 8:
        print ('----------AA----------')
        Matrix_similarity = similarity_indicators.AA.AA(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 9:
        print ('----------RA----------')
        Matrix_similarity = similarity_indicators.RA.RA(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 10:
        print ('----------SIM----------基于路径----------SIM----------')
        print ('----------LP----------')
        Matrix_similarity = similarity_indicators.LP.LP(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 11:
        print ('----------Katz----------')
        Matrix_similarity = similarity_indicators.Katz.Katz(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 12:
        print ('----------SIM----------基于随机游走----------SIM----------')
        print ('----------ACT----------')
        Matrix_similarity = similarity_indicators.ACT.ACT(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 13:
        print ('----------Cos----------')
        Matrix_similarity = similarity_indicators.Cos.Cos(MatrixAdjacency_Train)
        Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    else:
        print ("Method Error!")
        
similarity_EndTime = time.clock()
print ('----------！！----------')
print ("All SimilarityTime:", similarity_EndTime- similarity_StartTime)

# #计算auc
Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)

endTime = time.clock()
print ("\nRunTime:", endTime - startTime)


import similarity_indicators.CommonNeighbor
import similarity_indicators.Salton
import similarity_indicators.Jaccard
import similarity_indicators.Sorenson
import similarity_indicators.HPI
import similarity_indicators.HDI
import similarity_indicators.LHN_I
import similarity_indicators.PA
import similarity_indicators.AA
import similarity_indicators.RA
import similarity_indicators.RWR
def LPmethod(MatrixAdjacency_Train,Method):
    
    
    if Method == 0:
        
        print ('----------Cn----------')
        Matrix_similarity = similarity_indicators.CommonNeighbor.Cn(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 1:
        print ('----------Salton----------')
        Matrix_similarity = similarity_indicators.Salton.Salton(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 2:
        print ('----------Jaccard----------')
        Matrix_similarity = similarity_indicators.Jaccard.Jaccavrd(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 3:
        print ('----------Sorenson----------')
        Matrix_similarity = similarity_indicators.Sorenson.Sorenson(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 4:
        print ('----------HPI----------')
        Matrix_similarity = similarity_indicators.HPI.HPI(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 5:
        print ('----------HDI----------')
        Matrix_similarity = similarity_indicators.HDI.HDI(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 6:
        print ('----------LHN-1----------')
        Matrix_similarity = similarity_indicators.LHN_I.LHN_I(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 7:
        print ('----------PA----------')
        Matrix_similarity = similarity_indicators.PA.PA(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 8:
        print ('----------AA----------')
        Matrix_similarity = similarity_indicators.AA.AA(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 9:
        print ('----------RA----------')
        Matrix_similarity = similarity_indicators.RA.RA(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 10:
       
        print( '----------LP----------')
        Matrix_similarity = similarity_indicators.LP.LP(MatrixAdjacency_Train)
        #valuation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 11:
        print ('----------Katz----------')
        Matrix_similarity = similarity_indicators.Katz.Katz(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 12:
        
        print ('----------ACT----------')
        Matrix_similarity = similarity_indicators.ACT.ACT(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 13:
        print ('----------Cos----------')
        Matrix_similarity = similarity_indicators.Cos.Cos(MatrixAdjacency_Train)
        #Evaluation_Indicators.AUC.Calculation_AUC(MatrixAdjacency_Train, MatrixAdjacency_Test, Matrix_similarity, MaxNodeNum)
    elif Method == 14:
        print ('----------RWR----------')
        Matrix_similarity = similarity_indicators.RWR.RWR(MatrixAdjacency_Train)
        
    
    else:
        print ("Method Error!")
            
            
    return Matrix_similarity



import copy
import math

AALetter = [
    "A",
    "R",
    "N",
    "D",
    "C",
    "E",
    "Q",
    "G",
    "H",
    "I",
    "L",
    "K",
    "M",
    "F",
    "P",
    "S",
    "T",
    "W",
    "Y",
    "V",
]

_Hydrophobicity = {"1": "RKEDQN", "2": "GASTPHY", "3": "CLVIMFW"}
# '1'stand for Polar; '2'stand for Neutral, '3' stand for Hydrophobicity

_NormalizedVDWV = {"1": "GASTPD", "2": "NVEQIL", "3": "MHKFRYW"}
# '1'stand for (0-2.78); '2'stand for (2.95-4.0), '3' stand for (4.03-8.08)

_Polarity = {"1": "LIFWCMVY", "2": "CPNVEQIL", "3": "KMHFRYW"}
# '1'stand for (4.9-6.2); '2'stand for (8.0-9.2), '3' stand for (10.4-13.0)

_Charge = {"1": "KR", "2": "ANCQGHILMFPSTWYV", "3": "DE"}
# '1'stand for Positive; '2'stand for Neutral, '3' stand for Negative

_SecondaryStr = {"1": "EALMQKRH", "2": "VIYCWFT", "3": "GNPSD"}
# '1'stand for Helix; '2'stand for Strand, '3' stand for coil

_SolventAccessibility = {"1": "ALFCGIVW", "2": "RKQEND", "3": "MPSTHY"}
# '1'stand for Buried; '2'stand for Exposed, '3' stand for Intermediate

_Polarizability = {"1": "GASDT", "2": "CPNVEQIL", "3": "KMHFRYW"}
# '1'stand for (0-0.108); '2'stand for (0.128-0.186), '3' stand for (0.219-0.409)


# You can continuely add other properties of AADs to compute descriptors of
# protein sequence.

_AATProperty = (
    _Hydrophobicity,
    _NormalizedVDWV,
    _Polarity,
    _Charge,
    _SecondaryStr,
    _SolventAccessibility,
    _Polarizability,
)

_AATPropertyName = (
    "_Hydrophobicity",
    "_NormalizedVDWV",
    "_Polarity",
    "_Charge",
    "_SecondaryStr",
    "_SolventAccessibility",
    "_Polarizability",
)


def StringtoNum(ProteinSequence, AAProperty):
    """
    Tranform the protein sequence into the string form such as 32123223132121123.

    Usage:

    result=StringtoNum(protein,AAProperty)

    Input: protein is a pure protein sequence.

    AAProperty is a dict form containing classifciation of amino acids such as _Polarizability.

    Output: result is a string such as 123321222132111123222
    """
    hardProteinSequence = copy.deepcopy(ProteinSequence)
    for k, m in AAProperty.items():
        for index in m:
            hardProteinSequence = hardProteinSequence.replace(index, k)
    TProteinSequence = hardProteinSequence

    return TProteinSequence


def CalculateComposition(ProteinSequence, AAProperty, AAPName):
    """
    ###############################################################################################
    A method used for computing composition descriptors.

    Usage:

    result=CalculateComposition(protein,AAProperty,AAPName)

    Input: protein is a pure protein sequence.

    AAProperty is a dict form containing classifciation of amino acids such as _Polarizability.

    AAPName is a string used for indicating a AAP name.

    Output: result is a dict form containing composition descriptors based on the given property.
    ###############################################################################################
    """
    TProteinSequence = StringtoNum(ProteinSequence, AAProperty)
    Result = {}
    Num = len(TProteinSequence)
    Result[AAPName + "C" + "1"] = round(float(TProteinSequence.count("1")) / Num, 3)
    Result[AAPName + "C" + "2"] = round(float(TProteinSequence.count("2")) / Num, 3)
    Result[AAPName + "C" + "3"] = round(float(TProteinSequence.count("3")) / Num, 3)
    return Result


def CalculateTransition(ProteinSequence, AAProperty, AAPName):
    """
    ###############################################################################################
    A method used for computing transition descriptors

    Usage:

    result=CalculateTransition(protein,AAProperty,AAPName)

    Input:protein is a pure protein sequence.

    AAProperty is a dict form containing classifciation of amino acids such as _Polarizability.

    AAPName is a string used for indicating a AAP name.

    Output:result is a dict form containing transition descriptors based on the given property.
    ###############################################################################################
    """

    TProteinSequence = StringtoNum(ProteinSequence, AAProperty)
    Result = {}
    Num = len(TProteinSequence)
    CTD = TProteinSequence
    Result[AAPName + "T" + "12"] = round(
        float(CTD.count("12") + CTD.count("21")) / (Num - 1), 3
    )
    Result[AAPName + "T" + "13"] = round(
        float(CTD.count("13") + CTD.count("31")) / (Num - 1), 3
    )
    Result[AAPName + "T" + "23"] = round(
        float(CTD.count("23") + CTD.count("32")) / (Num - 1), 3
    )
    return Result


def CalculateDistribution(ProteinSequence, AAProperty, AAPName):
    """
    ###############################################################################################
    A method used for computing distribution descriptors.

    Usage:

    result=CalculateDistribution(protein,AAProperty,AAPName)

    Input:protein is a pure protein sequence.

    AAProperty is a dict form containing classifciation of amino acids such as _Polarizability.

    AAPName is a string used for indicating a AAP name.

    Output:result is a dict form containing Distribution descriptors based on the given property.
    ###############################################################################################
    """
    TProteinSequence = StringtoNum(ProteinSequence, AAProperty)
    Result = {}
    Num = len(TProteinSequence)
    for i in ("1", "2", "3"):
        num = TProteinSequence.count(i)
        ink = 1
        indexk = 0
        cds = []
        while ink <= num:
            indexk = TProteinSequence.find(i, indexk) + 1
            cds.append(indexk)
            ink = ink + 1

        if cds == []:
            Result[AAPName + "D" + i + "001"] = 0
            Result[AAPName + "D" + i + "025"] = 0
            Result[AAPName + "D" + i + "050"] = 0
            Result[AAPName + "D" + i + "075"] = 0
            Result[AAPName + "D" + i + "100"] = 0
        else:
            Result[AAPName + "D" + i + "001"] = round(float(cds[0]) / Num * 100, 3)
            Result[AAPName + "D" + i + "025"] = round(
                float(cds[int(math.floor(num * 0.25)) - 1]) / Num * 100, 3
            )
            Result[AAPName + "D" + i + "050"] = round(
                float(cds[int(math.floor(num * 0.5)) - 1]) / Num * 100, 3
            )
            Result[AAPName + "D" + i + "075"] = round(
                float(cds[int(math.floor(num * 0.75)) - 1]) / Num * 100, 3
            )
            Result[AAPName + "D" + i + "100"] = round(float(cds[-1]) / Num * 100, 3)

    return Result


def CalculateCompositionHydrophobicity(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating composition descriptors based on Hydrophobicity of

    AADs.

    Usage:

    result=CalculateCompositionHydrophobicity(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Composition descriptors based on Hydrophobicity.
    ###############################################################################################
    """

    result = CalculateComposition(ProteinSequence, _Hydrophobicity, "_Hydrophobicity")
    return result


def CalculateCompositionNormalizedVDWV(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating composition descriptors based on NormalizedVDWV of

    AADs.

    Usage:

    result=CalculateCompositionNormalizedVDWV(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Composition descriptors based on NormalizedVDWV.
    ###############################################################################################
    """
    result = CalculateComposition(ProteinSequence, _NormalizedVDWV, "_NormalizedVDWV")
    return result


def CalculateCompositionPolarity(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating composition descriptors based on Polarity of

    AADs.

    Usage:

    result=CalculateCompositionPolarity(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Composition descriptors based on Polarity.
    ###############################################################################################
    """

    result = CalculateComposition(ProteinSequence, _Polarity, "_Polarity")
    return result


def CalculateCompositionCharge(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating composition descriptors based on Charge of

    AADs.

    Usage:

    result=CalculateCompositionCharge(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Composition descriptors based on Charge.
    ###############################################################################################
    """

    result = CalculateComposition(ProteinSequence, _Charge, "_Charge")
    return result


def CalculateCompositionSecondaryStr(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating composition descriptors based on SecondaryStr of

    AADs.

    Usage:

    result=CalculateCompositionSecondaryStr(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Composition descriptors based on SecondaryStr.
    ###############################################################################################
    """

    result = CalculateComposition(ProteinSequence, _SecondaryStr, "_SecondaryStr")
    return result


def CalculateCompositionSolventAccessibility(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating composition descriptors based on SolventAccessibility

    of  AADs.

    Usage:

    result=CalculateCompositionSolventAccessibility(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Composition descriptors based on SolventAccessibility.
    ###############################################################################################
    """

    result = CalculateComposition(
        ProteinSequence, _SolventAccessibility, "_SolventAccessibility"
    )
    return result


def CalculateCompositionPolarizability(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating composition descriptors based on Polarizability of

    AADs.

    Usage:

    result=CalculateCompositionPolarizability(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Composition descriptors based on Polarizability.
    ###############################################################################################
    """

    result = CalculateComposition(ProteinSequence, _Polarizability, "_Polarizability")
    return result


##################################################################################################


##################################################################################################
def CalculateTransitionHydrophobicity(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Transition descriptors based on Hydrophobicity of

    AADs.

    Usage:

    result=CalculateTransitionHydrophobicity(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Transition descriptors based on Hydrophobicity.
    ###############################################################################################
    """

    result = CalculateTransition(ProteinSequence, _Hydrophobicity, "_Hydrophobicity")
    return result


def CalculateTransitionNormalizedVDWV(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Transition descriptors based on NormalizedVDWV of

    AADs.

    Usage:

    result=CalculateTransitionNormalizedVDWV(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Transition descriptors based on NormalizedVDWV.
    ###############################################################################################
    """

    result = CalculateTransition(ProteinSequence, _NormalizedVDWV, "_NormalizedVDWV")
    return result


def CalculateTransitionPolarity(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Transition descriptors based on Polarity of

    AADs.

    Usage:

    result=CalculateTransitionPolarity(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Transition descriptors based on Polarity.
    ###############################################################################################
    """

    result = CalculateTransition(ProteinSequence, _Polarity, "_Polarity")
    return result


def CalculateTransitionCharge(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Transition descriptors based on Charge of

    AADs.

    Usage:

    result=CalculateTransitionCharge(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Transition descriptors based on Charge.
    ###############################################################################################
    """

    result = CalculateTransition(ProteinSequence, _Charge, "_Charge")
    return result


def CalculateTransitionSecondaryStr(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Transition descriptors based on SecondaryStr of

    AADs.

    Usage:

    result=CalculateTransitionSecondaryStr(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Transition descriptors based on SecondaryStr.
    ###############################################################################################
    """

    result = CalculateTransition(ProteinSequence, _SecondaryStr, "_SecondaryStr")
    return result


def CalculateTransitionSolventAccessibility(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Transition descriptors based on SolventAccessibility

    of  AADs.

    Usage:

    result=CalculateTransitionSolventAccessibility(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Transition descriptors based on SolventAccessibility.
    ###############################################################################################
    """

    result = CalculateTransition(
        ProteinSequence, _SolventAccessibility, "_SolventAccessibility"
    )
    return result


def CalculateTransitionPolarizability(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Transition descriptors based on Polarizability of

    AADs.

    Usage:

    result=CalculateTransitionPolarizability(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Transition descriptors based on Polarizability.
    ###############################################################################################
    """

    result = CalculateTransition(ProteinSequence, _Polarizability, "_Polarizability")
    return result


##################################################################################################
##################################################################################################
def CalculateDistributionHydrophobicity(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Distribution descriptors based on Hydrophobicity of

    AADs.

    Usage:

    result=CalculateDistributionHydrophobicity(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Distribution descriptors based on Hydrophobicity.
    ###############################################################################################
    """

    result = CalculateDistribution(ProteinSequence, _Hydrophobicity, "_Hydrophobicity")
    return result


def CalculateDistributionNormalizedVDWV(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Distribution descriptors based on NormalizedVDWV of

    AADs.

    Usage:

    result=CalculateDistributionNormalizedVDWV(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Distribution descriptors based on NormalizedVDWV.
    ###############################################################################################
    """

    result = CalculateDistribution(ProteinSequence, _NormalizedVDWV, "_NormalizedVDWV")
    return result


def CalculateDistributionPolarity(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Distribution descriptors based on Polarity of

    AADs.

    Usage:

    result=CalculateDistributionPolarity(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Distribution descriptors based on Polarity.
    ###############################################################################################
    """

    result = CalculateDistribution(ProteinSequence, _Polarity, "_Polarity")
    return result


def CalculateDistributionCharge(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Distribution descriptors based on Charge of

    AADs.

    Usage:

    result=CalculateDistributionCharge(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Distribution descriptors based on Charge.
    ###############################################################################################
    """

    result = CalculateDistribution(ProteinSequence, _Charge, "_Charge")
    return result


def CalculateDistributionSecondaryStr(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Distribution descriptors based on SecondaryStr of

    AADs.

    Usage:

    result=CalculateDistributionSecondaryStr(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Distribution descriptors based on SecondaryStr.
    ###############################################################################################
    """

    result = CalculateDistribution(ProteinSequence, _SecondaryStr, "_SecondaryStr")
    return result


def CalculateDistributionSolventAccessibility(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Distribution descriptors based on SolventAccessibility

    of  AADs.

    Usage:

    result=CalculateDistributionSolventAccessibility(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Distribution descriptors based on SolventAccessibility.
    ###############################################################################################
    """

    result = CalculateDistribution(
        ProteinSequence, _SolventAccessibility, "_SolventAccessibility"
    )
    return result


def CalculateDistributionPolarizability(ProteinSequence):
    """
    ###############################################################################################
    A method used for calculating Distribution descriptors based on Polarizability of

    AADs.

    Usage:

    result=CalculateDistributionPolarizability(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing Distribution descriptors based on Polarizability.
    ###############################################################################################
    """

    result = CalculateDistribution(ProteinSequence, _Polarizability, "_Polarizability")
    return result


##################################################################################################


def CalculateC(ProteinSequence):
    """
    ###############################################################################################
    Calculate all composition descriptors based seven different properties of AADs.

    Usage:

    result=CalculateC(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing all composition descriptors.
    ###############################################################################################
    """
    result = {}
    result.update(CalculateCompositionPolarizability(ProteinSequence))
    result.update(CalculateCompositionSolventAccessibility(ProteinSequence))
    result.update(CalculateCompositionSecondaryStr(ProteinSequence))
    result.update(CalculateCompositionCharge(ProteinSequence))
    result.update(CalculateCompositionPolarity(ProteinSequence))
    result.update(CalculateCompositionNormalizedVDWV(ProteinSequence))
    result.update(CalculateCompositionHydrophobicity(ProteinSequence))
    return result


def CalculateT(ProteinSequence):
    """
    ###############################################################################################
    Calculate all transition descriptors based seven different properties of AADs.

    Usage:

    result=CalculateT(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing all transition descriptors.
    ###############################################################################################
    """
    result = {}
    result.update(CalculateTransitionPolarizability(ProteinSequence))
    result.update(CalculateTransitionSolventAccessibility(ProteinSequence))
    result.update(CalculateTransitionSecondaryStr(ProteinSequence))
    result.update(CalculateTransitionCharge(ProteinSequence))
    result.update(CalculateTransitionPolarity(ProteinSequence))
    result.update(CalculateTransitionNormalizedVDWV(ProteinSequence))
    result.update(CalculateTransitionHydrophobicity(ProteinSequence))
    return result


def CalculateD(ProteinSequence):
    """
    ###############################################################################################
    Calculate all distribution descriptors based seven different properties of AADs.

    Usage:

    result=CalculateD(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing all distribution descriptors.
    ###############################################################################################
    """
    result = {}
    result.update(CalculateDistributionPolarizability(ProteinSequence))
    result.update(CalculateDistributionSolventAccessibility(ProteinSequence))
    result.update(CalculateDistributionSecondaryStr(ProteinSequence))
    result.update(CalculateDistributionCharge(ProteinSequence))
    result.update(CalculateDistributionPolarity(ProteinSequence))
    result.update(CalculateDistributionNormalizedVDWV(ProteinSequence))
    result.update(CalculateDistributionHydrophobicity(ProteinSequence))
    return result


def CalculateCTD(ProteinSequence):
    """
    ###############################################################################################
    Calculate all CTD descriptors based seven different properties of AADs.

    Usage:

    result=CalculateCTD(protein)

    Input:protein is a pure protein sequence.

    Output:result is a dict form containing all CTD descriptors.
    ###############################################################################################
    """
    result = {}
    result.update(CalculateCompositionPolarizability(ProteinSequence))
    result.update(CalculateCompositionSolventAccessibility(ProteinSequence))
    result.update(CalculateCompositionSecondaryStr(ProteinSequence))
    result.update(CalculateCompositionCharge(ProteinSequence))
    result.update(CalculateCompositionPolarity(ProteinSequence))
    result.update(CalculateCompositionNormalizedVDWV(ProteinSequence))
    result.update(CalculateCompositionHydrophobicity(ProteinSequence))
    result.update(CalculateTransitionPolarizability(ProteinSequence))
    result.update(CalculateTransitionSolventAccessibility(ProteinSequence))
    result.update(CalculateTransitionSecondaryStr(ProteinSequence))
    result.update(CalculateTransitionCharge(ProteinSequence))
    result.update(CalculateTransitionPolarity(ProteinSequence))
    result.update(CalculateTransitionNormalizedVDWV(ProteinSequence))
    result.update(CalculateTransitionHydrophobicity(ProteinSequence))
    result.update(CalculateDistributionPolarizability(ProteinSequence))
    result.update(CalculateDistributionSolventAccessibility(ProteinSequence))
    result.update(CalculateDistributionSecondaryStr(ProteinSequence))
    result.update(CalculateDistributionCharge(ProteinSequence))
    result.update(CalculateDistributionPolarity(ProteinSequence))
    result.update(CalculateDistributionNormalizedVDWV(ProteinSequence))
    result.update(CalculateDistributionHydrophobicity(ProteinSequence))
    return result


##################################################################################################

if __name__ == "__main__":
    protein = "ADGCGVGEGTGQGPMCNCMCMKWVYADEDAADLESDSFADEDASLESDSFPWSNQRVFCSFADEDAS"
    print(CalculateCTD(protein))



import tensorflow as tf
import numpy as np
from .graph import *
from . import walker
import time
import ast


def fc_op(input_op, name, n_out, layer_collector, act_func=tf.nn.leaky_relu):
    n_in = input_op.get_shape()[-1].value
    with tf.name_scope(name) as scope:
        kernel = tf.Variable(tf.contrib.layers.xavier_initializer()([n_in, n_out]), dtype=tf.float32, name=scope + "w")

        # kernel = tf.Variable(tf.random_normal([n_in, n_out]))
        biases = tf.Variable(tf.constant(0, shape=[1, n_out], dtype=tf.float32), name=scope + 'b')

        fc = tf.add(tf.matmul(input_op, kernel), biases)
        activation = act_func(fc, name=scope + 'act')
        layer_collector.append([kernel, biases])
        return activation


class SDNE(object):
    def __init__(self, graph, encoder_layer_list, alpha=1e-6, beta=5., nu1=1e-5, nu2=1e-4,
                 batch_size=200, epoch=100, learning_rate=None):
        """
        encoder_layer_list: a list of numbers of the neuron at each ecdoer layer, the last number is the
        dimension of the output node representation
        Eg:
        if node size is 2000, encoder_layer_list=[1000, 128], then the whole neural network would be
        2000(input)->1000->128->1000->2000, SDNE extract the middle layer as the node representation
        """
        self.g = graph

        self.node_size = self.g.G.number_of_nodes()
        self.dim = encoder_layer_list[-1]

        self.encoder_layer_list = [self.node_size]
        self.encoder_layer_list.extend(encoder_layer_list)
        self.encoder_layer_num = len(encoder_layer_list)+1

        self.alpha = alpha
        self.beta = beta
        self.nu1 = nu1
        self.nu2 = nu2
        self.bs = batch_size
        self.epoch = epoch
        self.max_iter = (epoch * self.node_size) // batch_size

        self.lr = learning_rate
        if self.lr is None:
            self.lr = tf.train.inverse_time_decay(0.03, self.max_iter, decay_steps=1, decay_rate=0.9999)

        self.sess = tf.Session()
        self.vectors = {}

        self.adj_mat = self.getAdj()
        self.embeddings = self.train()

        look_back = self.g.look_back_list

        for i, embedding in enumerate(self.embeddings):
            self.vectors[look_back[i]] = embedding

    def getAdj(self):
        node_size = self.g.node_size
        look_up = self.g.look_up_dict
        adj = np.zeros((node_size, node_size))
        for edge in self.g.G.edges():
            adj[look_up[edge[0]]][look_up[edge[1]]] = self.g.G[edge[0]][edge[1]]['weight']
        return adj

    def train(self):
        adj_mat = self.adj_mat

        AdjBatch = tf.placeholder(tf.float32, [None, self.node_size], name='adj_batch')
        Adj = tf.placeholder(tf.float32, [None, None], name='adj_mat')
        B = tf.placeholder(tf.float32, [None, self.node_size], name='b_mat')

        fc = AdjBatch
        scope_name = 'encoder'
        layer_collector = []

        with tf.name_scope(scope_name):
            for i in range(1, self.encoder_layer_num):
                fc = fc_op(fc,
                           name=scope_name+str(i),
                           n_out=self.encoder_layer_list[i],
                           layer_collector=layer_collector)

        _embeddings = fc

        scope_name = 'decoder'
        with tf.name_scope(scope_name):
            for i in range(self.encoder_layer_num-2, 0, -1):
                fc = fc_op(fc,
                           name=scope_name+str(i),
                           n_out=self.encoder_layer_list[i],
                           layer_collector=layer_collector)
            fc = fc_op(fc,
                       name=scope_name+str(0),
                       n_out=self.encoder_layer_list[0],
                       layer_collector=layer_collector,)

        _embeddings_norm = tf.reduce_sum(tf.square(_embeddings), 1, keepdims=True)

        L_1st = tf.reduce_sum(
            Adj * (
                    _embeddings_norm - 2 * tf.matmul(
                        _embeddings, tf.transpose(_embeddings)
                    ) + tf.transpose(_embeddings_norm)
            )
        )

        L_2nd = tf.reduce_sum(tf.square((AdjBatch - fc) * B))

        L = L_2nd + self.alpha * L_1st

        for param in layer_collector:
            L += self.nu1 * tf.reduce_sum(tf.abs(param[0])) + self.nu2 * tf.reduce_sum(tf.square(param[0]))

        optimizer = tf.train.AdamOptimizer(self.lr)

        train_op = optimizer.minimize(L)

        init = tf.global_variables_initializer()
        self.sess.run(init)

        # print("total iter: %i" % self.max_iter)
        for step in range(self.max_iter):
            index = np.random.randint(self.node_size, size=self.bs)
            adj_batch_train = adj_mat[index, :]
            adj_mat_train = adj_batch_train[:, index]
            b_mat_train = np.ones_like(adj_batch_train)
            b_mat_train[adj_batch_train != 0] = self.beta

            self.sess.run(train_op, feed_dict={AdjBatch: adj_batch_train,
                                               Adj: adj_mat_train,
                                               B: b_mat_train})
            if step % 50 == 0:
                l, l1, l2 = self.sess.run((L, L_1st, L_2nd),
                                          feed_dict={AdjBatch: adj_batch_train,
                                                     Adj: adj_mat_train,
                                                     B: b_mat_train})
                # print("step %i: total loss: %s, l1 loss: %s, l2 loss: %s" % (step, l, l1, l2))

        return self.sess.run(_embeddings, feed_dict={AdjBatch: adj_mat})

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors)
        fout.write("{} {}\n".format(node_num, self.dim))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node, ' '.join([str(x) for x in vec])))
        fout.close()


class SDNE2(object):
    def __init__(self, graph, encoder_layer_list, alpha=1e-6, beta=5., nu1=1e-5, nu2=1e-5,
                 batch_size=100, max_iter=2000, learning_rate=None):

        self.g = graph

        self.node_size = self.g.G.number_of_nodes()
        self.rep_size = encoder_layer_list[-1]

        self.encoder_layer_list = [self.node_size] + encoder_layer_list
        self.encoder_layer_num = len(encoder_layer_list)+1

        self.alpha = alpha
        self.beta = beta
        self.nu1 = nu1
        self.nu2 = nu2
        self.bs = batch_size
        self.max_iter = max_iter
        self.lr = learning_rate
        if self.lr is None:
            self.lr = tf.train.inverse_time_decay(0.1, self.max_iter, decay_steps=1, decay_rate=0.9999)

        self.sess = tf.Session()
        self.vectors = {}

        self.adj_mat = self.getAdj()
        self.deg_vec = np.sum(self.adj_mat, axis=1)
        self.embeddings = self.get_train()

        look_back = self.g.look_back_list

        for i, embedding in enumerate(self.embeddings):
            self.vectors[look_back[i]] = embedding

    def getAdj(self):
        node_size = self.g.node_size
        look_up = self.g.look_up_dict
        adj = np.zeros((node_size, node_size))
        for edge in self.g.G.edges():
            adj[look_up[edge[0]]][look_up[edge[1]]] = self.g.G[edge[0]][edge[1]]['weight']
        return adj

    def model(self, node, layer_collector, scope_name):
        fc = node
        with tf.name_scope(scope_name + 'encoder'):
            for i in range(1, self.encoder_layer_num):
                fc = fc_op(fc,
                           name=scope_name+str(i),
                           n_out=self.encoder_layer_list[i],
                           layer_collector=layer_collector)

        _embeddings = fc

        with tf.name_scope(scope_name + 'decoder'):
            for i in range(self.encoder_layer_num-2, -1, -1):
                fc = fc_op(fc,
                           name=scope_name+str(i),
                           n_out=self.encoder_layer_list[i],
                           layer_collector=layer_collector)

        return _embeddings, fc

    def generate_batch(self, shuffle=True):
        adj = self.adj_mat

        row_indices, col_indices = adj.nonzero()
        sample_index = np.arange(row_indices.shape[0])
        num_of_batches = row_indices.shape[0] // self.bs
        counter = 0
        if shuffle:
            np.random.shuffle(sample_index)

        while True:
            batch_index = sample_index[self.bs * counter:self.bs * (counter + 1)]

            nodes_a = adj[row_indices[batch_index], :]
            nodes_b = adj[col_indices[batch_index], :]
            weights = adj[row_indices[batch_index], col_indices[batch_index]]
            weights = np.reshape(weights, [-1, 1])

            beta_mask_a = np.ones_like(nodes_a)
            beta_mask_a[nodes_a != 0] = self.beta
            beta_mask_b = np.ones_like(nodes_b)
            beta_mask_b[nodes_b != 0] = self.beta

            if counter == num_of_batches:
                counter = 0
                np.random.shuffle(sample_index)
            else:
                counter += 1

            yield (nodes_a, nodes_b, beta_mask_a, beta_mask_b, weights)

    def get_train(self):

        NodeA = tf.placeholder(tf.float32, [None, self.node_size], name='node_a')
        BmaskA = tf.placeholder(tf.float32, [None, self.node_size], name='beta_mask_a')
        NodeB = tf.placeholder(tf.float32, [None, self.node_size], name='node_b')
        BmaskB = tf.placeholder(tf.float32, [None, self.node_size], name='beta_mask_b')
        Weights = tf.placeholder(tf.float32, [None, 1], name='adj_weights')

        layer_collector = []
        nodes = tf.concat([NodeA, NodeB], axis=0)
        bmasks = tf.concat([BmaskA, BmaskB], axis=0)
        emb, recons = self.model(nodes, layer_collector, 'reconstructor')
        embs = tf.split(emb, num_or_size_splits=2, axis=0)

        L_1st = tf.reduce_sum(Weights * (tf.reduce_sum(tf.square(embs[0] - embs[1]), axis=1)))

        L_2nd = tf.reduce_sum(tf.square((nodes - recons) * bmasks))

        L = L_2nd + self.alpha * L_1st

        for param in layer_collector:
            L += self.nu1 * tf.reduce_sum(tf.abs(param[0])) + self.nu2 * tf.reduce_sum(tf.square(param[0]))

        # lr = tf.train.exponential_decay(1e-6, self.max_iter, decay_steps=1, decay_rate=0.9999)
        # optimizer = tf.train.MomentumOptimizer(lr, 0.99, use_nesterov=True)

        optimizer = tf.train.AdamOptimizer(self.lr)
        train_op = optimizer.minimize(L)

        init = tf.global_variables_initializer()
        self.sess.run(init)

        generator = self.generate_batch()

        for step in range(self.max_iter+1):
            nodes_a, nodes_b, beta_mask_a, beta_mask_b, weights = generator.__next__()

            feed_dict = {NodeA: nodes_a,
                         NodeB: nodes_b,
                         BmaskA: beta_mask_a,
                         BmaskB: beta_mask_b,
                         Weights: weights}

            self.sess.run(train_op, feed_dict=feed_dict)
            # if step % 50 == 0:
            #     # print("step %i: %s" % (step, self.sess.run([L, L_1st, L_2nd], feed_dict=feed_dict)))

        return self.sess.run(emb, feed_dict={NodeA: self.adj_mat[0:1, :], NodeB: self.adj_mat[1:, :]})

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors)
        fout.write("{} {}\n".format(node_num, self.rep_size))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node, ' '.join([str(x) for x in vec])))
        fout.close()


from utils import *
from Classifier.Auto import AutoClassifier
data = ReadCsv('feature.csv')
targets = ReadCsv('labels.csv')
targets = [int(i[0]) for i in targets]
data, targets=shuffle(data, targets)

cv = StratifiedKFold(n_splits=5)
aucs=[]
for train, test in cv.split(data, targets):

      
        model = AutoClassifier.fit(data[train], targets[train])
        Testdata=pd.DataFrame(data[test])
        y_prob = model.predict_proba(Testdata)
        y_prob = [i[1] for i in y_prob.values.tolist()]
      
        fpr, tpr, thresholds = roc_curve(targets[test], y_prob)
        roc_auc = auc(fpr, tpr)
        aucs.append(roc_auc)
roc_auc=np.mean(aucs) 
print("AUC=",roc_auc)



#coding=UTF-8
'''
Created on Nov 29, 2016

@author: ZWT
'''
import numpy as np
import time

def Katz(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
    Parameter = 0.01
    Matrix_EYE = np.eye(MatrixAdjacency_Train.shape[0])
    Temp = Matrix_EYE - MatrixAdjacency_Train * Parameter
    
    Matrix_similarity = np.linalg.inv(Temp)

    Matrix_similarity = Matrix_similarity - Matrix_EYE

    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity



# -*- coding: utf-8 -*-

import geatpy as ea 
import numpy as np
import geatpy as ea
from sklearn import svm
from sklearn import preprocessing
from sklearn.model_selection import cross_val_score
from multiprocessing.dummy import Pool as ThreadPool
class MyProblem(ea.Problem): # 继承Problem父类
    def __init__(self,datas,data_targets,optimize_type):
        name = 'MyProblem' # 初始化name（函数名称，可以随意设置）
        M = 1 # 初始化M（目标维数）
        maxormins = [-1] # 初始化maxormins（目标最小最大化标记列表，1：最小化该目标；-1：最大化该目标）
        Dim = 2 # 初始化Dim（决策变量维数）
        varTypes = [0, 0] # 初始化varTypes（决策变量的类型，元素为0表示对应的变量是连续的；1表示是离散的）
        lb = [2**(-8)] * Dim # 决策变量下界
        ub = [2**8] * Dim # 决策变量上界
        lbin = [1] * Dim # 决策变量下边界（0表示不包含该变量的下边界，1表示包含）
        ubin = [1] * Dim # 决策变量上边界（0表示不包含该变量的上边界，1表示包含）
        # 调用父类构造方法完成实例化
        ea.Problem.__init__(self, name, M, maxormins, Dim, varTypes, lb, ub, lbin, ubin)
        # 目标函数计算中用到的一些数据
        
        self.data = datas # 训练集的特征数据（归一化）
        self.dataTarget = data_targets
        self.optimize_type = optimize_type
    
    def aimFunc(self, pop): # 目标函数，采用多线程加速计算
        Vars = pop.Phen # 得到决策变量矩阵
        pop.ObjV = np.zeros((pop.sizes, 1)) # 初始化种群个体目标函数值列向量
        def subAimFunc(i):
            C = Vars[i, 0]
            G = Vars[i, 1]
            svc = svm.SVC(C=C, kernel='rbf', gamma=G).fit(self.data, self.dataTarget) # 创建分类器对象并用训练集的数据拟合分类器模型
            scores = cross_val_score(svc, self.data, self.dataTarget, scoring=self.optimize_type,cv=5) # 计算交叉验证的得分
            pop.ObjV[i] = scores.mean() # 把交叉验证的平均得分作为目标函数值
        pool = ThreadPool(2) # 设置池的大小
        pool.map(subAimFunc, list(range(pop.sizes)))
    
    def test(self, C, G): # 代入优化后的C、Gamma对测试集进行检验
        # 读取测试集数据
        fp = open('iris_test.data')
        datas = []
        data_targets = []
        for line in fp.readlines():
            line_data = line.strip('\n').split(',')
            data = []
            for i in line_data[0:4]:
                data.append(float(i))
            datas.append(data)
            data_targets.append(line_data[4])
        fp.close()
        data_test = preprocessing.scale(np.array(datas)) # 测试集的特征数据（归一化）
        dataTarget_test = np.array(data_targets) # 测试集的标签数据
        svc = svm.SVC(C=C, kernel='rbf', gamma=G).fit(self.data, self.dataTarget) # 创建分类器对象并用训练集的数据拟合分类器模型
        dataTarget_predict = svc.predict(data_test) # 采用训练好的分类器对象对测试集数据进行预测
        print("测试集数据分类正确率 = %s%%"%(len(np.where(dataTarget_predict == dataTarget_test)[0]) / len(dataTarget_test) * 100))




def Best_Param(datas,data_targets,optimize_type):


    """===============================实例化问题对象==========================="""
    problem = MyProblem(datas,data_targets,optimize_type) # 生成问题对象


    """=================================种群设置==============================="""
    Encoding = 'RI'       # 编码方式
    NIND = 2             # 种群规模
    Field = ea.crtfld(Encoding, problem.varTypes, problem.ranges, problem.borders) # 创建区域描述器
    population = ea.Population(Encoding, Field, NIND) # 实例化种群对象（此时种群还没被初始化，仅仅是完成种群对象的实例化）
    """===============================算法参数设置============================="""
    myAlgorithm = ea.soea_DE_rand_1_bin_templet(problem, population) # 实例化一个算法模板对象
    myAlgorithm.MAXGEN = 10 # 最大进化代数
    myAlgorithm.trappedValue = 1e-6 # “进化停滞”判断阈值
    myAlgorithm.maxTrappedCount = 10 # 进化停滞计数器最大上限值，如果连续maxTrappedCount代被判定进化陷入停滞，则终止进化
    myAlgorithm.logTras = 1  # 设置每隔多少代记录日志，若设置成0则表示不记录日志
    myAlgorithm.verbose = True  # 设置是否打印输出日志信息
    myAlgorithm.drawing = 1  # 设置绘图方式（0：不绘图；1：绘制结果图；2：绘制目标空间过程动画；3：绘制决策空间过程动画）
    """===========================调用算法模板进行种群进化======================="""
    [BestIndi, population] = myAlgorithm.run()  # 执行算法模板，得到最优个体以及最后一代种群
    BestIndi.save()  # 把最优个体的信息保存到文件中
    """==================================输出结果============================="""
    print('Timed：%f s' % myAlgorithm.passTime)
    
    if BestIndi.sizes != 0:
        print('The optimal objective function value is:%s' % BestIndi.ObjV[0][0])
        print('The optimal parameter value is:')
        for i in range(BestIndi.Phen.shape[1]):
            print(BestIndi.Phen[0, i])
    else:
        print('No feasible solution was found.')


    best_param={'C':BestIndi.Phen[0][0],
                'gamma':BestIndi.Phen[0][1],
                'cache_size':3000,
               }
    return best_param


from Embedding.sdne import *
def Sdne(input_file, output_file, matrix,alpha=1e-6, beta=5., nu1=1e-5, nu2=1e-4, batch_size=200, epoch=5, learning_rate=0.01):

    np.savetxt(input_file,matrix,fmt='%d')
    t1 = time.time()
    g = Graph()
    print("Reading...")


    g.read_edgelist(filename=input_file, weighted=False, directed=False)
    encoder_layer_list = [1000, 128]
    model = SDNE(g, encoder_layer_list=encoder_layer_list,
                alpha=alpha, beta=beta, nu1=nu1, nu2=nu2,
                batch_size=batch_size, epoch=epoch, learning_rate=learning_rate)


    t2 = time.time()
    print(t2-t1)
    print("Saving embeddings...")
    model.save_embeddings(output_file)
    Embedding=np.loadtxt(output_file, dtype=float, skiprows=1)
    return Embedding



from datetime import time
from autobiotool.utils import *
import sklearn
from autogluon.tabular import  TabularPredictor
random.seed(123)

def fit(data, targets):
    
    label=len(data[0])
    save_path = './model/auto/' 
    y_train=[[i] for i in targets]
    Train=np.hstack((data,y_train))
    train_data = pd.DataFrame(Train)
    model = TabularPredictor(label=label, path=save_path).fit(train_data)
    
    return model




from time import time
import networkx as nx
import numpy as np
import scipy.io as sio
import scipy.sparse as sp
import scipy.sparse.linalg as lg
from . import graph as g
from sklearn.preprocessing import normalize
from .graph import *
from . import walker



class LLE(object):

    def __init__(self, graph, d):
        ''' Initialize the LocallyLinearEmbedding class

        Args:
          graph: nx.DiGraph
            input Graph
          d: int
            dimension of the embedding
        '''

        self._d = d
        self._method_name = "lle_svd"
        self.g = graph
        self._node_num = graph.node_size
        self.learn_embedding()

    def learn_embedding(self):
        graph = self.g.G
        graph = graph.to_undirected()
        t1 = time()
        A = nx.to_scipy_sparse_matrix(graph)
        # print(np.sum(A.todense(), axis=0))
        normalize(A, norm='l1', axis=1, copy=False)
        I_n = sp.eye(graph.number_of_nodes())
        I_min_A = I_n - A
        # print(I_min_A)
        u, s, vt = lg.svds(I_min_A, k=self._d + 1, which='SM')
        t2 = time()
        self._X = vt.T
        self._X = self._X[:, 1:]
        return self._X, (t2 - t1)
        # I_n = sp.eye(graph.number_of_nodes())

    @property
    def vectors(self):
        vectors = {}
        look_back = self.g.look_back_list
        for i, embedding in enumerate(self._X):
            vectors[look_back[i]] = embedding
        return vectors

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors.keys())
        fout.write("{} {}\n".format(node_num, self._d))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node,
                                        ' '.join([str(x) for x in vec])))
        fout.close()



from __future__ import print_function
import math
import numpy as np
from numpy import linalg as la
from sklearn.preprocessing import normalize
from .gcn.utils import *


class TADW(object):

    def __init__(self, graph, dim, lamb=0.2):
        self.g = graph
        self.lamb = lamb
        self.dim = int(dim/2)
        self.train()

    def getAdj(self):
        graph = self.g.G
        node_size = self.g.node_size
        look_up = self.g.look_up_dict
        adj = np.zeros((node_size, node_size))
        for edge in self.g.G.edges():
            adj[look_up[edge[0]]][look_up[edge[1]]] = 1.0
            adj[look_up[edge[1]]][look_up[edge[0]]] = 1.0
        # ScaleSimMat
        return adj/np.sum(adj, axis=1)

    def save_embeddings(self, filename):
        fout = open(filename, 'w')
        node_num = len(self.vectors.keys())
        fout.write("{} {}\n".format(node_num, self.dim*2))
        for node, vec in self.vectors.items():
            fout.write("{} {}\n".format(node, ' '.join([str(x) for x in vec])))
        fout.close()

    def getT(self):
        g = self.g.G
        look_back = self.g.look_back_list
        self.features = np.vstack([g.nodes[look_back[i]]['feature']
                                   for i in range(g.number_of_nodes())])
        self.preprocessFeature()
        return self.features.T

    def preprocessFeature(self):
        if self.features.shape[1] > 200:
            U, S, VT = la.svd(self.features)
            Ud = U[:, 0:200]
            Sd = S[0:200]
            self.features = np.array(Ud)*Sd.reshape(200)

    def train(self):
        self.adj = self.getAdj()
        # M=(A+A^2)/2 where A is the row-normalized adjacency matrix
        self.M = (self.adj + np.dot(self.adj, self.adj))/2
        # T is feature_size*node_num, text features
        self.T = self.getT()
        self.node_size = self.adj.shape[0]
        self.feature_size = self.features.shape[1]
        self.W = np.random.randn(self.dim, self.node_size)
        self.H = np.random.randn(self.dim, self.feature_size)
        # Update
        for i in range(20):
            # print('Iteration ', i)
            # Update W
            B = np.dot(self.H, self.T)
            drv = 2 * np.dot(np.dot(B, B.T), self.W) - \
                2*np.dot(B, self.M.T) + self.lamb*self.W
            Hess = 2*np.dot(B, B.T) + self.lamb*np.eye(self.dim)
            drv = np.reshape(drv, [self.dim*self.node_size, 1])
            rt = -drv
            dt = rt
            vecW = np.reshape(self.W, [self.dim*self.node_size, 1])
            while np.linalg.norm(rt, 2) > 1e-4:
                dtS = np.reshape(dt, (self.dim, self.node_size))
                Hdt = np.reshape(np.dot(Hess, dtS), [
                                 self.dim*self.node_size, 1])

                at = np.dot(rt.T, rt)/np.dot(dt.T, Hdt)
                vecW = vecW + at*dt
                rtmp = rt
                rt = rt - at*Hdt
                bt = np.dot(rt.T, rt)/np.dot(rtmp.T, rtmp)
                dt = rt + bt * dt
            self.W = np.reshape(vecW, (self.dim, self.node_size))

            # Update H
            drv = np.dot((np.dot(np.dot(np.dot(self.W, self.W.T), self.H), self.T)
                          - np.dot(self.W, self.M.T)), self.T.T) + self.lamb*self.H
            drv = np.reshape(drv, (self.dim*self.feature_size, 1))
            rt = -drv
            dt = rt
            vecH = np.reshape(self.H, (self.dim*self.feature_size, 1))
            while np.linalg.norm(rt, 2) > 1e-4:
                dtS = np.reshape(dt, (self.dim, self.feature_size))
                Hdt = np.reshape(np.dot(np.dot(np.dot(self.W, self.W.T), dtS), np.dot(self.T, self.T.T))
                                 + self.lamb*dtS, (self.dim*self.feature_size, 1))
                at = np.dot(rt.T, rt)/np.dot(dt.T, Hdt)
                vecH = vecH + at*dt
                rtmp = rt
                rt = rt - at*Hdt
                bt = np.dot(rt.T, rt)/np.dot(rtmp.T, rtmp)
                dt = rt + bt * dt
            self.H = np.reshape(vecH, (self.dim, self.feature_size))
        self.Vecs = np.hstack(
            (normalize(self.W.T), normalize(np.dot(self.T.T, self.H.T))))
        # get embeddings
        self.vectors = {}
        look_back = self.g.look_back_list
        for i, embedding in enumerate(self.Vecs):
            self.vectors[look_back[i]] = embedding



#coding=UTF-8
'''
Created on 2016��10��25��

@author: ZWT
'''
import csv
import numpy as np
import random

method = raw_input("请输入1选择ProbS推荐，输入2选择HeatS推荐:")
Re_length = int(raw_input("请输入所需推荐列表的长度:"))
timeNum = int(raw_input("清输入AUC循环次数:"))

TrainSet_CsvFile = u"data/votes.csv"
ScoialSet_CsvFIle = u'data/social.csv'
ProbeSet_CsvFile = u"data/probeSet.csv"

TrainSet_userList = []#训练集user列表
TrainSet_itemList = []#训练集item列表
ProbeSet_userList = []#测试集user列表
ProbeSet_itemList = []#测试集item列表


######训练集TrainSet_CsvFile文件转邻接矩阵
TrainSet = csv.reader(file(TrainSet_CsvFile,'rb'))
TrainSetData = []
for line in TrainSet:
    TrainSet_userList.append(int(line[1])-1)
    TrainSet_itemList.append(int(line[2])-1)
    TrainSetData.append([int(line[1]),int(line[2])])
TrainSet_userList = set(TrainSet_userList)
TrainSet_itemList = set(TrainSet_itemList)
Matrix_Adjacency = np.zeros([len(TrainSet_userList),len(TrainSet_itemList)])
for n in range(len(TrainSetData)):
    i = int(TrainSetData[n][0]) - 1
    j = int(TrainSetData[n][1]) - 1
    Matrix_Adjacency[i,j] = 1
print "邻接矩阵转换："+str(Matrix_Adjacency.shape)

######社交网络ScoialSet_CsvFile文件求社交网络矩阵
ScoialSet = csv.reader(file(ScoialSet_CsvFIle,'rb'))
ScoialSetData = []
ScoialSet_rowList = []
ScoialSet_colList = []
for line in ScoialSet:
    ScoialSet_rowList.append(int(line[0]))
    ScoialSet_colList.append(int(line[1]))
    ScoialSetData.append([int(line[0]),int(line[1])])
ScoialSet_rowList = set(ScoialSet_rowList)
ScoialSet_colList = set(ScoialSet_colList)
Matrix_Scoial = np.zeros([len(ScoialSet_rowList),len(ScoialSet_colList)])
for n in range(len(ScoialSetData)):
    i = int(ScoialSetData[n][0]) - 1
    j = int(ScoialSetData[n][1]) - 1
    Matrix_Scoial[i,j] = 1
print "社交矩阵转换："+str(Matrix_Scoial.shape)

######测试集ProbeSet_CsvFile文件求用户喜欢列表
ProbeSet = csv.reader(file(ProbeSet_CsvFile,'rb'))
ProbeSetData = []
for line in ProbeSet:
    ProbeSet_userList.append(int(line[1])-1)
    ProbeSet_itemList.append(int(line[2])-1)
    ProbeSetData.append([int(line[1]),int(line[2])])
ProbeSet_userList = set(ProbeSet_userList)
ProbeSet_itemList = set(ProbeSet_itemList)

######邻接矩阵求转置矩阵
Matrix_Transposed = Matrix_Adjacency.T
print "转置矩阵转换："+str(Matrix_Transposed.shape)
    
######邻接矩阵和转置矩阵求user的度和item的度，转为对角矩阵再求逆矩阵
degree_user = [sum(Matrix_Adjacency[i]) for i in range(len(Matrix_Adjacency))]
degree_item = [sum(Matrix_Transposed[i]) for i in range(len(Matrix_Transposed))]
degree_scoial = [sum(Matrix_Scoial[i]) for i in range(len(Matrix_Scoial))]

diag_user = np.diag(degree_user)
print "user的度的对角矩阵计算："+str(diag_user.shape)
inv_diag_user = np.linalg.inv(diag_user)
print "user的度的对角矩阵求逆矩阵："+str(inv_diag_user.shape)
diag_item = np.diag(degree_item)
print "item的度的对角矩阵计算："+str(diag_item.shape)
inv_diag_item = np.linalg.inv(diag_item)
print "item的度的对角矩阵求逆矩阵："+str(inv_diag_item.shape)
diag_scoial = np.diag(degree_scoial)
print "scoial的度的对角矩阵计算："+str(diag_item.shape)
inv_diag_scoial = np.linalg.inv(diag_scoial)
print "scoial的度的对角矩阵求逆矩阵："+str(inv_diag_item.shape)

######邻接矩阵，转置矩阵，逆矩阵求物质网络矩阵
if method == "1":
    ProbS_UserItem = np.dot(np.dot(np.dot(Matrix_Transposed,inv_diag_user),Matrix_Adjacency),inv_diag_item)
    print "物质UserItem矩阵计算："+str(ProbS_UserItem.shape)
    ProbS_Scoial = np.dot(np.dot(np.dot(np.dot(np.dot(Matrix_Transposed,inv_diag_user),Matrix_Scoial),inv_diag_scoial),Matrix_Adjacency),inv_diag_item)
    print "物质Scoial矩阵计算："+str(ProbS_Scoial.shape)
    Matrix_Network_UserItem = ProbS_UserItem
    Matrix_Network_Scoial = ProbS_Scoial
elif method == "2":
    HeatS_UserItem = np.dot(np.dot(np.dot(inv_diag_item,Matrix_Transposed),inv_diag_user),Matrix_Adjacency)
    print "热能UserItem矩阵计算："+str(HeatS_UserItem.shape)
    HeatS_Scoial = np.dot(np.dot(np.dot(np.dot(np.dot(inv_diag_item,Matrix_Transposed),Matrix_Scoial),inv_diag_scoial),inv_diag_user),Matrix_Adjacency)
    print "热能Scoial矩阵计算："+str(HeatS_Scoial.shape)
    Matrix_Network_UserItem = HeatS_UserItem
    Matrix_Network_Scoial = HeatS_Scoial
else:
    print "推荐算法选择错误！输入1选择ProbS，输入2选择HeatS"

######计算质量分配
QualityList = []

Params = 1.0
count = 1
while count < 11:
    print "\n"+'这是第 '+str(count)+" 次计算，参数为："+str(Params)
    Precision_List = []
    Recall_List = []
    RSgrade_List = []
    AUCgrade_List =[]

    for user in range(len(TrainSet_userList)):
        #print "USER_ID为 "+str(user)+" 推荐："
        ####user的推荐列表
        userQualityList = []
        Matrix_user_F = Matrix_Adjacency[user]
        Matrix_Wuzhi_UserItem = Params * (np.dot(Matrix_Network_UserItem,Matrix_user_F))
        Matrix_Wuzhi_Scoial = (1-Params) * (np.dot(Matrix_Network_Scoial,Matrix_user_F))
        Matrix_Wuzhi = Matrix_Wuzhi_UserItem + Matrix_Wuzhi_Scoial
        
        for item in range(len(Matrix_Adjacency[user])):
            if (Matrix_Adjacency[user,item] == 0):
                userQualityList.append([item,Matrix_Wuzhi[item],0])
            else:
                userQualityList.append([item,Matrix_Wuzhi[item],1])
        userQualityList = sorted(userQualityList,key = lambda userQualityList:userQualityList[1],reverse=True)
        #按物质量从大到小排序
        userRecommendList = []
        userRsRecommendList  = []

        for item in userQualityList:
            if item[2] == 0:
                userRecommendList.append(item)
                userRsRecommendList.append(item)
        if userRecommendList[Re_length-1][1]==userRecommendList[Re_length][1]:
            tempList = [index for index,x in enumerate(userRecommendList) if x[1]==userRecommendList[Re_length-1][1]]
            randomList = random.sample(tempList,(Re_length - tempList[0]))#randomList存储随机的物质相同的item下标
            userRecommendList = userRecommendList[0:tempList[0]] + [userRecommendList[i]for i in randomList]#截取前段物质不同的列表，加上随机划分的物质相同的列表
        else:
            userRecommendList = userRecommendList[0:(Re_length)]
#         print "USER_ID为"+str(user)+"的推荐列表如下:"
#         print userRecommendList
        
        ####user的喜欢列表
        userLikeLsit = []
        for data in ProbeSetData:
            if int(data[0]) == int(user)+1:
                userLikeLsit.append(data[1])
#         print "USER_ID为"+str(user)+"的喜欢列表如下:"
#         print userLikeLsit
        ####user的购买列表
        userbuylist = []
        for data in TrainSetData:
            if int(data[0] == int(user)+1):
                userbuylist.append(data[1])
#         print "USER_ID为"+str(user)+"的购买列表如下:"
#         print userbuylist
        #计算user召回率        #计算user推荐准确率
        if userLikeLsit:        
            userLikeNum = 0
            userRecallList = []
            userRSgrade = []
            userAUCgrade = []
            
            userLikeQualityList = []
            
            userLikeRsList = [] 
            for likeID in userLikeLsit:
                for reList in userRecommendList:
                    if int(likeID)-1 == int(reList[0]):
                        userRecallList.append(likeID)
                        userLikeNum += 1
                for rsQuList in userRsRecommendList:
                    if int(likeID)-1 == int(rsQuList[0]):
                        tempLikeRsList = [int(index)+1 for index,x in enumerate(userRsRecommendList) if x[1] == rsQuList[1]]
    #                     print '该item'+str(likeID)+'的排名列表'+str(tempLikeRsList)
                        userLikeRsList.append((float(sum(tempLikeRsList))/len(tempLikeRsList))/len(userLikeLsit))
    #                     print '该item'+str(likeID)+'的平均排名'+str(float(sum(tempLikeRsList))/len(tempLikeRsList))
                likeIDQuality = [userQualityList[index] for index,x in enumerate(userQualityList) if x[0] == likeID-1][0]
                userLikeQualityList.append(likeIDQuality)
#             print "USER_ID为"+str(user)+"的喜欢物质量列表为如下:"    
#             print  userLikeQualityList
#             print "USER_ID为"+str(user)+"的召回列表为如下:"
#             print userRecallList
            userPrecision = float(userLikeNum)/Re_length
#             print "USER_ID为"+str(user)+"的推荐准确率为:"+str(userPrecision)
            Precision_List.append(userPrecision)
            userRecall = float(userLikeNum)/len(userLikeLsit)
#             print "USER_ID为"+str(user)+"的推荐召回率为:"+str(userRecall)
            Recall_List.append(userRecall)
#             print "该用户不相关的item长度为："+str(len(TrainSet_itemList)-len(userbuylist))
            userRSgrade = float(sum(userLikeRsList))/(len(TrainSet_itemList)-len(userbuylist))
#             print "USER_ID为"+str(user)+"的排名RS为:"+str(userRSgrade)
            RSgrade_List.append(userRSgrade)
        
            #计算AUC
            AUCgrade = 0
            for time in range(timeNum):
                randomUserQuality = random.choice(userQualityList)
                randomUserLike = random.choice(userLikeQualityList)
    #             randomLikeID = random.choice(userLikeLsit)
    #             randomUserLike = [userQualityList[index] for index,x in enumerate(userQualityList) if x[0]== randomLikeID][0]
                if randomUserQuality[1] > randomUserLike[1]:
                    AUCgrade += 0
                elif randomUserQuality[1] < randomUserLike[1]:
                    AUCgrade += 1
                else:
                    AUCgrade += 0.5
            userAUCgrade= float(AUCgrade)/timeNum
#             print "USER_ID为"+str(user)+"的推荐AUC为:"+str(userAUCgrade)
            AUCgrade_List.append(userAUCgrade)
#         else:
#             print "USER_ID没有喜欢列表"
        #user += 1
    count += 1
    Params = float(Params) - 0.1
        
    Precision = sum(Precision_List)/len(ProbeSet_userList)
    print "推荐算法的平均准确率是："+str(Precision)    
    Recall = sum(Recall_List)/len(ProbeSet_userList)
    print "推荐算法的平均召回率是："+str(Recall)
    F1 = float(2*Precision*Recall)/(Precision+Recall)
    print "推荐算法的F1指标为："+str(F1)
    RS = sum(RSgrade_List)/len(ProbeSet_userList)
    print "推荐算法的推荐准确度是："+str(RS)
    AUCgrade = sum(AUCgrade_List)/len(ProbeSet_userList)
    print "推荐算法的推荐AUC是："+str(AUCgrade)





from __future__ import division
from __future__ import print_function

import time
import tensorflow as tf

from gcn.utils import *
from gcn.models import GCN, MLP

# Set random seed
seed = 123
np.random.seed(seed)
tf.set_random_seed(seed)

# Settings
flags = tf.app.flags
FLAGS = flags.FLAGS
# 'cora', 'citeseer', 'pubmed'
flags.DEFINE_string('dataset', 'cora', 'Dataset string.')
# 'gcn', 'gcn_cheby', 'dense'
flags.DEFINE_string('model', 'gcn', 'Model string.')
flags.DEFINE_float('learning_rate', 0.01, 'Initial learning rate.')
flags.DEFINE_integer('epochs', 200, 'Number of epochs to train.')
flags.DEFINE_integer('hidden1', 16, 'Number of units in hidden layer 1.')
flags.DEFINE_float('dropout', 0.5, 'Dropout rate (1 - keep probability).')
flags.DEFINE_float('weight_decay', 5e-4,
                   'Weight for L2 loss on embedding matrix.')
flags.DEFINE_integer('early_stopping', 10,
                     'Tolerance for early stopping (# of epochs).')
flags.DEFINE_integer('max_degree', 3, 'Maximum Chebyshev polynomial degree.')

# Load data
adj, features, y_train, y_val, y_test, train_mask, val_mask, test_mask = load_data(
    FLAGS.dataset)

# Some preprocessing
features = preprocess_features(features)
if FLAGS.model == 'gcn':
    support = [preprocess_adj(adj)]
    num_supports = 1
    model_func = GCN
elif FLAGS.model == 'gcn_cheby':
    support = chebyshev_polynomials(adj, FLAGS.max_degree)
    num_supports = 1 + FLAGS.max_degree
    model_func = GCN
elif FLAGS.model == 'dense':
    support = [preprocess_adj(adj)]  # Not used
    num_supports = 1
    model_func = MLP
else:
    raise ValueError('Invalid argument for model: ' + str(FLAGS.model))

# Define placeholders
placeholders = {
    'support': [tf.sparse_placeholder(tf.float32) for _ in range(num_supports)],
    'features': tf.sparse_placeholder(tf.float32, shape=tf.constant(features[2], dtype=tf.int64)),
    'labels': tf.placeholder(tf.float32, shape=(None, y_train.shape[1])),
    'labels_mask': tf.placeholder(tf.int32),
    'dropout': tf.placeholder_with_default(0., shape=()),
    # helper variable for sparse dropout
    'num_features_nonzero': tf.placeholder(tf.int32)
}

# Create model
model = model_func(placeholders, input_dim=features[2][1], logging=True)

# Initialize session
sess = tf.Session()


# Define model evaluation function
def evaluate(features, support, labels, mask, placeholders):
    t_test = time.time()
    feed_dict_val = construct_feed_dict(
        features, support, labels, mask, placeholders)
    outs_val = sess.run([model.loss, model.accuracy], feed_dict=feed_dict_val)
    return outs_val[0], outs_val[1], (time.time() - t_test)


# Init variables
sess.run(tf.global_variables_initializer())

cost_val = []

# Train model
for epoch in range(FLAGS.epochs):

    t = time.time()
    # Construct feed dictionary
    feed_dict = construct_feed_dict(
        features, support, y_train, train_mask, placeholders)
    feed_dict.update({placeholders['dropout']: FLAGS.dropout})

    # Training step
    outs = sess.run([model.opt_op, model.loss, model.accuracy],
                    feed_dict=feed_dict)

    # Validation
    cost, acc, duration = evaluate(
        features, support, y_val, val_mask, placeholders)
    cost_val.append(cost)

    # Print results
    print("Epoch:", '%04d' % (epoch + 1), "train_loss=", "{:.5f}".format(outs[1]),
          "train_acc=", "{:.5f}".format(
        outs[2]), "val_loss=", "{:.5f}".format(cost),
        "val_acc=", "{:.5f}".format(acc), "time=", "{:.5f}".format(time.time() - t))

    if epoch > FLAGS.early_stopping and cost_val[-1] > np.mean(cost_val[-(FLAGS.early_stopping+1):-1]):
        print("Early stopping...")
        break

print("Optimization Finished!")

# Testing
test_cost, test_acc, test_duration = evaluate(
    features, support, y_test, test_mask, placeholders)
print("Test set results:", "cost=", "{:.5f}".format(test_cost),
      "accuracy=", "{:.5f}".format(test_acc), "time=", "{:.5f}".format(test_duration))



from __future__ import print_function
import numpy
from sklearn.multiclass import OneVsRestClassifier
from sklearn.metrics import f1_score
from sklearn.preprocessing import MultiLabelBinarizer
from time import time


class TopKRanker(OneVsRestClassifier):
    def predict(self, X, top_k_list):
        probs = numpy.asarray(super(TopKRanker, self).predict_proba(X))
        all_labels = []
        for i, k in enumerate(top_k_list):
            probs_ = probs[i, :]
            labels = self.classes_[probs_.argsort()[-k:]].tolist()
            probs_[:] = 0
            probs_[labels] = 1
            all_labels.append(probs_)
        return numpy.asarray(all_labels)


class Classifier(object):

    def __init__(self, vectors, clf):
        self.embeddings = vectors
        self.clf = TopKRanker(clf)
        self.binarizer = MultiLabelBinarizer(sparse_output=True)

    def train(self, X, Y, Y_all):
        self.binarizer.fit(Y_all)
        X_train = [self.embeddings[x] for x in X]
        Y = self.binarizer.transform(Y)
        self.clf.fit(X_train, Y)

    def evaluate(self, X, Y):
        top_k_list = [len(l) for l in Y]
        Y_ = self.predict(X, top_k_list)
        Y = self.binarizer.transform(Y)
        averages = ["micro", "macro", "samples", "weighted"]
        results = {}
        for average in averages:
            results[average] = f1_score(Y, Y_, average=average)
        # print('Results, using embeddings of dimensionality', len(self.embeddings[X[0]]))
        # print('-------------------')
        # print(results)
        return results
        # print('-------------------')

    def predict(self, X, top_k_list):
        X_ = numpy.asarray([self.embeddings[x] for x in X])
        Y = self.clf.predict(X_, top_k_list=top_k_list)
        return Y

    def split_train_evaluate(self, X, Y, train_precent, seed=0):
        state = numpy.random.get_state()

        training_size = int(train_precent * len(X))
        numpy.random.seed(seed)
        shuffle_indices = numpy.random.permutation(numpy.arange(len(X)))
        X_train = [X[shuffle_indices[i]] for i in range(training_size)]
        Y_train = [Y[shuffle_indices[i]] for i in range(training_size)]
        X_test = [X[shuffle_indices[i]] for i in range(training_size, len(X))]
        Y_test = [Y[shuffle_indices[i]] for i in range(training_size, len(X))]

        self.train(X_train, Y_train, Y)
        numpy.random.set_state(state)
        return self.evaluate(X_test, Y_test)


def load_embeddings(filename):
    fin = open(filename, 'r')
    node_num, size = [int(x) for x in fin.readline().strip().split()]
    vectors = {}
    while 1:
        l = fin.readline()
        if l == '':
            break
        vec = l.strip().split(' ')
        assert len(vec) == size+1
        vectors[vec[0]] = [float(x) for x in vec[1:]]
    fin.close()
    assert len(vectors) == node_num
    return vectors


def read_node_label(filename):
    fin = open(filename, 'r')
    X = []
    Y = []
    while 1:
        l = fin.readline()
        if l == '':
            break
        vec = l.strip().split(' ')
        X.append(vec[0])
        Y.append(vec[1:])
    fin.close()
    return X, Y



"""Graph utilities."""

# from time import time
import networkx as nx
import pickle as pkl
import numpy as np
import scipy.sparse as sp

__author__ = "Zhang Zhengyan"
__email__ = "zhangzhengyan14@mails.tsinghua.edu.cn"


class Graph(object):
    def __init__(self):
        self.G = None
        self.look_up_dict = {}
        self.look_back_list = []
        self.node_size = 0

    def encode_node(self):
        look_up = self.look_up_dict
        look_back = self.look_back_list
        for node in self.G.nodes():
            look_up[node] = self.node_size
            look_back.append(node)
            self.node_size += 1
            self.G.nodes[node]['status'] = ''

    def read_g(self, g):
        self.G = g
        self.encode_node()

    def read_adjlist(self, filename):
        """ Read graph from adjacency file in which the edge must be unweighted
            the format of each line: v1 n1 n2 n3 ... nk
            :param filename: the filename of input file
        """
        self.G = nx.read_adjlist(filename, create_using=nx.DiGraph())
        for i, j in self.G.edges():
            self.G[i][j]['weight'] = 1.0
        self.encode_node()

    def read_edgelist(self, filename, weighted=False, directed=False):
        self.G = nx.DiGraph()

        if directed:
            def read_unweighted(l):
                src, dst = l.split()
                self.G.add_edge(src, dst)
                self.G[src][dst]['weight'] = 1.0

            def read_weighted(l):
                src, dst, w = l.split()
                self.G.add_edge(src, dst)
                self.G[src][dst]['weight'] = float(w)
        else:
            def read_unweighted(l):
                src, dst = l.split()
                self.G.add_edge(src, dst)
                self.G.add_edge(dst, src)
                self.G[src][dst]['weight'] = 1.0
                self.G[dst][src]['weight'] = 1.0

            def read_weighted(l):
                src, dst, w = l.split()
                self.G.add_edge(src, dst)
                self.G.add_edge(dst, src)
                self.G[src][dst]['weight'] = float(w)
                self.G[dst][src]['weight'] = float(w)
        fin = open(filename, 'r')
        func = read_unweighted
        if weighted:
            func = read_weighted
        while 1:
            l = fin.readline()
            if l == '':
                break
            func(l)
        fin.close()
        self.encode_node()

    def read_node_label(self, filename):
        fin = open(filename, 'r')
        while 1:
            l = fin.readline()
            if l == '':
                break
            vec = l.split()
            self.G.nodes[vec[0]]['label'] = vec[1:]
        fin.close()

    def read_node_features(self, filename):
        fin = open(filename, 'r')
        for l in fin.readlines():
            vec = l.split()
            self.G.nodes[vec[0]]['feature'] = np.array(
                [float(x) for x in vec[1:]])
        fin.close()

    def read_node_status(self, filename):
        fin = open(filename, 'r')
        while 1:
            l = fin.readline()
            if l == '':
                break
            vec = l.split()
            self.G.nodes[vec[0]]['status'] = vec[1]  # train test valid
        fin.close()

    def read_edge_label(self, filename):
        fin = open(filename, 'r')
        while 1:
            l = fin.readline()
            if l == '':
                break
            vec = l.split()
            self.G[vec[0]][vec[1]]['label'] = vec[2:]
        fin.close()



#coding=UTF-8
'''
Created on 2016��11��24��

@author: ZWT
'''
import numpy as np
import time

def Sorenson(MatrixAdjacency_Train):
    similarity_StartTime = time.clock()
    
    Matrix_similarity = np.dot(MatrixAdjacency_Train,MatrixAdjacency_Train)

    deg_row = sum(MatrixAdjacency_Train)
    deg_row.shape = (deg_row.shape[0],1)
    deg_row_T = deg_row.T
    tempdeg = deg_row + deg_row_T
    
    Matrix_similarity = (2 * Matrix_similarity) / tempdeg
    
    similarity_EndTime = time.clock()
    print ("    SimilarityTime: %f s" % (similarity_EndTime- similarity_StartTime))
    return Matrix_similarity
# import numpy.matlib as matlab
# def Sorenson(MatrixAdjacency_Train):
#     Matrix_similarity = np.dot(MatrixAdjacency_Train,MatrixAdjacency_Train)
# 
#     Matrix_similarity = np.triu(Matrix_similarity, 1)
#     
#     deg_col = matlab.repmat((matlab.sum(MatrixAdjacency_Train,1)), 1, matlab.size(MatrixAdjacency_Train, 1))
#     print deg_col.shape
#     deg_col_T = deg_col.T
#     deg_col = deg_col_T + deg_col
#     deg_col = np.triu(deg_col)
#     
#     Matrix_similarity = (2 * Matrix_similarity) / deg_col 
# 
#     return Matrix_similarity